'''
As a part of ``cylp.python.pivots`` it implements Dantzig's
 Simplex dual pivot rule. Although it already exists in CLP,
for testing purposes we implement one in Python.
'''

from __future__ import print_function
import sys
import numpy as np
from operator import itemgetter
from random import shuffle
from math import floor
from .DualPivotPythonBase import DualPivotPythonBase
#from cylp.py.pivots import DantzigPivot


class DualDantzigPivot(DualPivotPythonBase):
    '''
    Dantzig's dual pivot rule implementation.

    .. _custom-dual-pivot-usage:

    **Usage**

    >>> from cylp.cy import CyClpSimplex
    >>> from cylp.py.pivots.DualDantzigPivot import DualDantzigPivot
    >>> from cylp.py.pivots.DualDantzigPivot import getMpsExample
    >>> # Get the path to a sample mps file
    >>> f = getMpsExample()
    >>> s = CyClpSimplex()
    >>> s.readMps(f)  # Returns 0 if OK
    0
    >>> pivot = DualDantzigPivot(s)
    >>> s.setDualPivotMethod(pivot)
    >>> s.dual()
    'optimal'
    >>> round(s.objectiveValue, 5)
    2520.57174

    '''

    def __init__(self, clpModel):
        self.dim = clpModel.nRows + clpModel.nCols
        self.clpModel = clpModel

    def pivotRow(self):
        model = self.clpModel
        nConstraints = model.nConstraints
        basicVarInds = model.basicVariables

        u = model.upper[basicVarInds]
        l = model.lower[basicVarInds]
        s = model.solution[basicVarInds]
        infeasibilities = np.maximum(s - u, l - s)

        m = max(infeasibilities)

        if m > model.primalTolerance:
            return np.argmax(infeasibilities)
        return -1

    def updateWeights(self, inp, spare, spare2, updatedColumn):
        model = self.clpModel
        pr = model.pivotRow()
        model.updateColumnFT(spare, updatedColumn)
        indices = updatedColumn.indices
        elements = updatedColumn.elements
        if updatedColumn.isInPackedMode:
            if pr in indices:
                ind = np.where(indices==pr)[0][0]
                return elements[ind]
        else:
            return elements[pr]
        return 0

    def updatePrimalSolution(self, primalUpdate, primalRatio, objectiveChange):
        model = self.clpModel
        nConstraints = model.nConstraints
        basicVarInds = model.basicVariables
        rowNumbers = primalUpdate.indices
        elements = primalUpdate.elements
        nElements = primalUpdate.nElements
        changeObj = 0

        sol = model.solution[basicVarInds[rowNumbers]]
        cost = model.cost[basicVarInds[rowNumbers]]

        if primalUpdate.isInPackedMode:
            change = primalRatio * elements[:nElements]
            model.solution[basicVarInds[rowNumbers]] -= change
        else:
            change = primalRatio * elements[rowNumbers]
            model.solution[basicVarInds[rowNumbers]] -= change

        changeObj = -np.dot(change, cost)
        primalUpdate.clear()
        objectiveChange[0] += changeObj

        return changeObj


def getMpsExample():
    import os
    import inspect
    import sys
    cylpDir = os.environ['CYLP_SOURCE_DIR']
    return os.path.join(cylpDir, 'cylp', 'input', 'p0033.mps')


if __name__ == "__main__":
    print(sys.argv)
    if len(sys.argv) == 1:
        import doctest
        doctest.testmod()
    else:
        from cylp.cy import CyClpSimplex
        from cylp.py.pivots import DualDantzigPivot
        s = CyClpSimplex()
        s.readMps(sys.argv[1])  # Returns 0 if OK
        pivot = DualDantzigPivot(s)
        s.setDualPivotMethod(pivot)
        s.dual()



from cython.operator cimport dereference as deref
cimport cpython.ref as cpy_ref
from cpython.ref cimport PyObject
from cylp.cy cimport CyClpSimplex
from cylp.cy.CyCoinIndexedVector cimport CppCoinIndexedVector
from cpython cimport Py_INCREF, Py_DECREF


cdef extern from "ClpPrimalColumnPivot.hpp":
    cdef cppclass CyClpPrimalColumnPivot "ClpPrimalColumnPivot":
        pass
    CyClpPrimalColumnPivot* new_CyClpPrimalColumnPivot \
                                    "new ClpPrimalColumnPivot" ()

cdef extern from "ClpFactorization.hpp":
    cdef cppclass ClpFactorization:
        int updateColumnTranspose (CppCoinIndexedVector * regionSparse,
                  CppCoinIndexedVector * regionSparse2)


cdef extern from "IClpPrimalColumnPivotBase.h":
    #cdef cppclass CoinIndexedVector:
    #   pass
    ctypedef int (*runPivotColumn_t)(void *obj,
                                     CppCoinIndexedVector*,
                                     CppCoinIndexedVector*,
                                     CppCoinIndexedVector*,
                                     CppCoinIndexedVector*,
                                     CppCoinIndexedVector*)
    ctypedef CyClpPrimalColumnPivot* (*runClone_t)(void *obj, bint copyData)
    ctypedef void (*runSaveWeights_t)(void* obj,
                                      CyClpSimplex.CppIClpSimplex* model,
                                      int mode)
    cdef cppclass CppClpPrimalColumnPivotBase:
        CyClpSimplex.CppIClpSimplex* model()
        void setModel(CyClpSimplex.CppIClpSimplex* m)
        CppClpPrimalColumnPivotBase(PyObject* obj,
                                    runPivotColumn_t runPivotColumn,
                                    runClone_t runClone,
                                    runSaveWeights_t runSaveWeights)

        int pivotColumn(CppCoinIndexedVector*,
                        CppCoinIndexedVector*,
                        CppCoinIndexedVector*,
                        CppCoinIndexedVector*,
                        CppCoinIndexedVector*)

        CyClpPrimalColumnPivot* clone(bint copyData)
        void saveWeights(CyClpSimplex.CppIClpSimplex * model,
                         int mode)


cdef int RunPivotColumn(void *ptr,
                        CppCoinIndexedVector* v1,
                        CppCoinIndexedVector* v2,
                        CppCoinIndexedVector* v3,
                        CppCoinIndexedVector* v4,
                        CppCoinIndexedVector* v5)

cdef CyClpPrimalColumnPivot* RunClone(void *ptr, bint copyData)


cdef void RunSaveWeights(void *ptr, CyClpSimplex.CppIClpSimplex * model,
                         int mode)


cdef class CyClpPrimalColumnPivotBase:
    cdef CppClpPrimalColumnPivotBase* CppSelf
    cdef CyClpSimplex.CyClpSimplex cyModel
    #cpdef cyModel
    cdef pivotColumn(self, CppCoinIndexedVector* v1,
                        CppCoinIndexedVector* v2,
                        CppCoinIndexedVector* v3,
                        CppCoinIndexedVector* v4,
                        CppCoinIndexedVector* v5)

    cdef CyClpPrimalColumnPivot * clone(self, bint copyData)
    cdef void saveWeights(self, CyClpSimplex.CppIClpSimplex * model,
                          int mode)
    cdef CyClpSimplex.CppIClpSimplex* model(self)
    cdef void setModel(self, CyClpSimplex.CppIClpSimplex* m)
    cdef double* getReducedCosts(self)



cimport numpy as np
from cpython.ref cimport PyObject
from cylp.cy.CyCgl cimport CyCglCutGenerator, CppCglCutGenerator
from cylp.cy.CyCbcNode cimport CyCbcNode, CppICbcNode
from cylp.cy.CyOsiSolverInterface cimport CppOsiSolverInterface, CyOsiSolverInterface
from cpython cimport Py_INCREF, Py_DECREF


cdef extern from "CbcCompareUser.hpp":
    cdef cppclass CppCbcCompareUser "CbcCompareUser":
        pass
    ctypedef int (*runTest_t)(void* instance, CppICbcNode* x,
                              CppICbcNode* y) except? -1
    ctypedef bint (*runNewSolution_t)(void*instance, CppICbcModel* model,
                       double objectiveAtContinuous,
                       int numberInfeasibilitiesAtContinuous) except? -1
    ctypedef int (*runEvery1000Nodes_t)(void* instance,
                            CppICbcModel* model, int numberNodes) except? -1
    bint equalityTest(CppICbcNode* x, CppICbcNode* y)


cdef extern from "ICbcModel.hpp":
    cdef cppclass CppICbcModel "ICbcModel":
        PyObject* getPrimalVariableSolution()

        int cbcMain()
        int getSolutionCount()
        int getNumberHeuristicSolutions()
        int getNodeCount()
        double getObjValue()
        double getBestPossibleObjValue()
        int numberObjects()
        void setNodeCompare(PyObject* obj, runTest_t runTest,
                            runNewSolution_t runNewSolution,
                            runEvery1000Nodes_t runEvery1000Nodes)
        void addCutGenerator(CppCglCutGenerator* generator,
                        int howOften,
                        char* name,
                        bint normal,
                        bint atSolution,
                        bint infeasible,
                        int howOftenInSub,
                        int whatDepth,
                        int whatDepthInSub)
        void branchAndBound(int doStatistics)
        int status()
        int secondaryStatus()
        bint isProvenOptimal()
        bint isProvenInfeasible()
        bint isInitialSolveProvenPrimalInfeasible()
        bint isInitialSolveProvenDualInfeasible()
        bint isInitialSolveProvenOptimal()
        bint isInitialSolveAbandoned()

        bint setIntegerTolerance(double value)
        double getIntegerTolerance()

        bint setMaximumSeconds(double value)
        double getMaximumSeconds()

        bint setMaximumNodes(int value)
        int getMaximumNodes()

        int getNumberThreads()
        void setNumberThreads(int value)

        void setLogLevel(int value)
        int logLevel()

        void setAllowableFractionGap(double value)
        double getAllowableFractionGap()

        void setAllowablePercentageGap(double value)
        double getAllowablePercentageGap()

        void setAllowableGap(double value)
        double getAllowableGap()

        bint setMaximumSolutions(int value)
        int getMaximumSolutions()
        
        CppOsiSolverInterface* solver()

cdef class CyCbcModel:
    cdef CppICbcModel* CppSelf
    cdef object cyLPModel
    cdef object clpModel
    cdef object cutGenerators
    cdef setCppSelf(self, CppICbcModel* cppmodel)
    cdef setClpModel(self, clpmodel)
    cpdef addCutGenerator(self, CyCglCutGenerator generator,
                        howOften=*, name=*, normal=*, atSolution=*,
                        infeasible=*, howOftenInSub=*, whatDepth=*,
                        whatDepthInSub=*)



import numpy as np
from operator import itemgetter
from random import shuffle
from cylp.cy import CyCoinIndexedVector
import random
from math import floor
from cylp.py.pivots import PositiveEdgePivot


class PositiveEdgeWolfePivot(PositiveEdgePivot):

    def __init__(self, clpModel, bucketSize=1, EPSILON=10 ** (-7)):
        PositiveEdgePivot.__init__(self, clpModel, bucketSize, EPSILON)
        self.banList = []

    def pivotColumnFirst(self, updates, spareRow1, spareRow2, spareCol1, spareCol2):
        '''Finds the first variable with an acceptable
        reduced costs searching in random buckets
        '''
        self.updateReducedCosts(updates, spareRow1, spareRow2, spareCol1, spareCol2)
        s = self.clpModel
        rc = s.getReducedCosts()
        dim = s.getNumRows() + s.getNumCols()
        nRows = s.getNumRows()

        tol = s.dualTolerance()

        if not self.initialized:
            self.initialize()

        xranges = self.xranges
        numberOfSegments = self.numberOfSegments
        segmentSize = self.segmentSize

        shuffle(xranges)

        bestFreeVarInd = -1
        bestVarInd = -1
        bestCompVarInd = -1

        bestFreeVarRC = 0
        bestVarRC = 0
        bestCompVarRC = 0

        goodVariableFound = 0

        for iSegment in range(numberOfSegments):
#           comp_ind = [i for i in range(dim) if not s.flagged(i) and \
#                     (rc[i] < -tol and \
#                     s.getStatus(i) == 3 and \
#                     self.isCompatible(i)) \
#                     or \
#                     (rc[i] > tol and \
#                     s.getStatus(i) == 2 and \
#                     self.isCompatible(i)) \
#                     or \
#                     (abs(rc[i]) > tol and \
#                     (s.getStatus(i) == 4 or s.getStatus(i) == 0) and \
#                     self.isCompatible(i))]

            for i in ranges[iSegment]:
                if s.flagged(i) or \
                            abs(rc[i]) < self.EPSILON or \
                            i in self.banList:
                    continue
                if s.getStatus(i) == 3:  # at lowerbound
                    if rc[i] < bestCompVarRC and self.isCompatible(i):
                        bestCompVarRC = rc[i]
                        bestCompVarInd = i
                        goodVariableFound += 1
                    if rc[i] < bestVarRC:
                        bestVarRC = rc[i]
                        bestVarInd = i
                        goodVariableFound += 1

                elif s.getStatus(i) == 2:  # at upperbound
                    r = -rc[i]
                    if r < bestCompVarRC and self.isCompatible(i):
                        bestCompVarRC = r
                        bestCompVarInd = i
                        goodVariableFound += 1
                    if r < bestVarRC:
                        bestVarRC = r
                        bestVarInd = i
                        goodVariableFound += 1
                # Free or Superbasic
                elif s.getStatus(i) == 0 or s.getStatus(i) == 4:
                    r = -abs(rc[i])
                    if r < bestCompVarRC and self.isCompatible(i):
                        bestCompVarRC = r
                        bestCompVarInd = i
                        goodVariableFound += 1
                    if r < bestFreeVarRC:
                        bestFreeVarRC = r
                        bestFreeVarInd = i
                        goodVariableFound += 1

            if goodVariableFound > 0:
                break

        l = [bestVarRC, bestFreeVarRC, bestCompVarRC]

        needUpdate = True
        if bestVarInd != -1:
            if bestFreeVarInd != -1:
                if 0.1 * bestVarRC > bestFreeVarRC:
                    bestVarRC = bestFreeVarRC
                    bestVarInd = bestFreeVarInd

            if bestCompVarInd != -1:
                if 0.4 * bestVarRC > bestCompVarRC:
                    bestVarRC = bestCompVarRC
                    bestVarInd = bestCompVarInd
                    needUpdate = False

        elif bestFreeVarInd != -1:
            if bestCompVarInd != -1 and 0.4 * bestFreeVarRC > bestCompVarRC:
                bestVarRC = bestCompVarRC
                bestVarInd = bestCompVarInd
                needUpdate = False
            else:
                bestVarRC = bestFreeVarRC
                bestVarInd = bestFreeVarInd

        if needUpdate:
            self.updateP()
            self.updateW()

        return bestVarInd

    def saveWeights(self, model, mode):
        self.clpModel = model

    def isPivotAcceptable(self):
        #TODO ComplementarityList can be defined in the current class
        s = self.clpModel
        cl = s.getComplementarityList()
        pivotRow = s.pivotRow()
        if pivotRow < 0:
            return 1

        pivotVariable = s.getPivotVariable()
        leavingVarIndex = pivotVariable[pivotRow]
        colInd = s.sequenceIn()

        if s.getStatus(cl[colInd]) == 1 and \
            cl[colInd] != leavingVarIndex:
            self.banList.append(colInd)

            #s.setFlagged(colInd)
            return 0

        #self.banList = np.zeros(self.dim, np.int)
        self.banList = []

        return 1



from cylp.cy.CyCglCutGeneratorBase cimport *
from cylp.cy.CyOsiSolverInterface cimport CppOsiSolverInterface, CyOsiSolverInterface
from cylp.cy.CyOsiCuts cimport CppOsiCuts, CyOsiCuts
from cylp.cy.CyCglTreeInfo cimport CppCglTreeInfo, CyCglTreeInfo
cimport cython.operator.dereference as deref
from cylp.cy.CyClpSimplex cimport CyClpSimplex, CppIClpSimplex
from cylp.cy.CyCoinIndexedVector cimport CyCoinIndexedVector
from cpython cimport Py_INCREF, Py_DECREF

cdef class CyCutGeneratorPythonBase(CyCglCutGeneratorBase):
    cdef generateCuts(self, CppOsiSolverInterface *si,
                                     CppOsiCuts *cs,
                                     CppCglTreeInfo info)
    cdef CppCglCutGenerator * clone(self)
    cdef object cutGeneratorObject
    cdef object cyLPModel




cdef extern from "CglTreeInfo.hpp":
    cdef cppclass CppCglTreeInfo "CglTreeInfo":
        pass
    CppCglTreeInfo *new_CppCglTreeInfo "new CglTreeInfo" ()

cdef class CyCglTreeInfo:
    cdef CppCglTreeInfo* CppSelf
    cdef setCppSelf(self, CppCglTreeInfo* s)



'''
    Module provide a facility to model a linear program.
    Although the current usage is to pass the resulting LP to CLP
    to solve it is independant of the solver.

    .. _modeling-usage:

    **Usage (single dimension, using CyLPModel indirectly)**

    >>> import numpy as np
    >>> from cylp.cy import CyClpSimplex
    >>> from cylp.py.modeling.CyLPModel import CyLPArray
    >>>
    >>> s = CyClpSimplex()
    >>>
    >>> # Add variables
    >>> x = s.addVariable('x', 3)
    >>> y = s.addVariable('y', 2)
    >>>
    >>> # Create coefficients and bounds
    >>> A = np.matrix([[1., 2., 0],[1., 0, 1.]])
    >>> B = np.matrix([[1., 0, 0], [0, 0, 1.]])
    >>> D = np.matrix([[1., 2.],[0, 1]])
    >>> a = CyLPArray([5, 2.5])
    >>> b = CyLPArray([4.2, 3])
    >>> x_u= CyLPArray([2., 3.5])
    >>>
    >>> # Add constraints
    >>> s += A * x <= a
    >>> s += 2 <= B * x + D * y <= b
    >>> s += y >= 0
    >>> s += 1.1 <= x[1:3] <= x_u
    >>>
    >>> # Set the objective function
    >>> c = CyLPArray([1., -2., 3.])
    >>> s.objective = c * x + 2 * y.sum()
    >>>
    >>> # Solve using primal Simplex
    >>> s.primal()
    'optimal'
    >>> s.primalVariableSolution['x']
    array([ 0.2,  2. ,  1.1])
    >>> s.primalVariableSolution['y']
    array([ 0. ,  0.9])
    >>> s += x[2] + y[1] >= 2.1
    >>> s.primal()
    'optimal'
    >>> s.primalVariableSolution['x']
    array([ 0. ,  2. ,  1.1])
    >>> s.primalVariableSolution['y']
    array([ 0.,  1.])

    **Usage (single dimension, using CyLPModel directly, depending on the
      size could be faster than the indirect approach)**

    >>> import numpy as np
    >>> from cylp.cy import CyClpSimplex
    >>> from cylp.py.modeling.CyLPModel import CyLPModel, CyLPArray
    >>>
    >>> model = CyLPModel()
    >>>
    >>> # Add variables
    >>> x = model.addVariable('x', 3)
    >>> y = model.addVariable('y', 2)
    >>>
    >>> # Create coefficients and bounds
    >>> A = np.matrix([[1., 2., 0],[1., 0, 1.]])
    >>> B = np.matrix([[1., 0, 0], [0, 0, 1.]])
    >>> D = np.matrix([[1., 2.],[0, 1]])
    >>> a = CyLPArray([5, 2.5])
    >>> b = CyLPArray([4.2, 3])
    >>> x_u= CyLPArray([2., 3.5])
    >>>
    >>> # Add constraints
    >>> model += A * x <= a
    >>> model += 2 <= B * x + D * y <= b
    >>> model += y >= 0
    >>> model += 1.1 <= x[1:3] <= x_u
    >>>
    >>> # Set the objective function
    >>> c = CyLPArray([1., -2., 3.])
    >>> model.objective = c * x + 2 * y.sum()
    >>>
    >>> # Create a CyClpSimplex instance from the model
    >>> s = CyClpSimplex(model)
    >>>
    >>> # Solve using primal Simplex
    >>> s.primal()
    'optimal'
    >>> s.primalVariableSolution['x']
    array([ 0.2,  2. ,  1.1])
    >>> s.primalVariableSolution['y']
    array([ 0. ,  0.9])
    >>> s += x[2] + y[1] >= 2.1
    >>> s.primal()
    'optimal'
    >>> s.primalVariableSolution['x']
    array([ 0. ,  2. ,  1.1])
    >>> s.primalVariableSolution['y']
    array([ 0.,  1.])

    **Usage (multi-dimensions, using CyLPModel indirectly)**

    >>> from cylp.cy import CyClpSimplex
    >>> from cylp.py.modeling.CyLPModel import CyLPArray
    >>> s = CyClpSimplex()
    >>> x = s.addVariable('x', (5, 3, 6))
    >>> s += 2 * x[2, :, 3].sum() + 3 * x[0, 1, :].sum() >= 5
    >>> s += 0 <= x <= 1
    >>> c = CyLPArray(range(18))
    >>> s.objective = c * x[2, :, :] + c * x[0, :, :]
    >>> s.primal()
    'optimal'
    >>> sol = s.primalVariableSolution['x']
    >>> abs(sol[0, 1, 0] - 1) <= 10**-6
    True
    >>> abs(sol[2, 0, 3] - 1) <= 10**-6
    True

'''

from __future__ import print_function
from functools import reduce

# Python 3 does not have long, only int
try:
    long
except NameError:
    long = int

from itertools import product
from copy import deepcopy
from operator import mul
import numpy as np
from scipy import sparse
from scipy.sparse import identity, lil_matrix
from cylp.py.utils.util import Ind, getMultiDimMatrixIndex, getTupleIndex

NUMBERS = (int, float, long, np.int64, np.int32, np.double)
def isNumber(n):
    return (isinstance(n, NUMBERS) or
            (isinstance(n, CyLPArray) and n.shape == ()))

# Increase max stack size from 8MB to 512MB
#resource.setrlimit(resource.RLIMIT_STACK, (2**5,-1))
#sys.setrecursionlimit(10**6)

#from cylp.py.utils.sparseUtil import sparseConcat
def I(n):
    '''
    Return a sparse identity matrix of size *n*
    '''
    if n <= 0:
        return None
    #return csc_matrixPlus(sparse.eye(n, n))
    return identity(n).tocsr()

def identitySub(var):
    '''
    Return a sparse row sub-matrix of the identity matrix
    for ``var``.
    '''
    n = var.dim
    if var.parent is not None:
        n = var.parent.dim
    if var.dim == 1:
        m = lil_matrix((1, n))
        m[0, var.indices[0]] = 1
        return m
    return I(n)[var.indices, :]

class CyLPExpr:
    __array_ufunc__ = None
    
    operators = ('>=', '<=', '==', '+', '-', '*', 'u-', 'sum')

    def __init__(self, opr='', left='', right=''):
        self.opr = opr
        self.left = left
        self.right = right
        self.expr = self
        #self.variables = []

    def __hash__(self):
        return id(self)

    def __repr__(self):
        s = '%s %s %s' % (self.left, self.right, self.opr)
        return s

    def __le__(self, other):
        v = CyLPExpr(opr="<=", right=other, left=self.expr)
        self.expr = v
        return v

    def __ge__(self, other):
        v = CyLPExpr(opr=">=", right=other, left=self.expr)
        self.expr = v
        return v

    def __eq__(self, other):
        # Check if both sides are CyLPVar, in which case a pythonic
        # comparison is meant (for dictionary keys,...)
        if (other is None):
            return False
        if isinstance(self, CyLPVar) and isinstance(other, CyLPVar):
            return id(self) == id(other)
            #return (str(self) == str(other))
        v = CyLPExpr(opr="==", right=other, left=self.expr)
        self.expr = v
        return v

    def __rmul__(self, other):
        if type(other)==np.ndarray:
            raise ValueError("You should use @ when multiplying a variable with a numpy array")
        v = CyLPExpr(opr="*", left=other, right=self)
        v.expr = v
        self.expr = v
        return v

    def __mul__(self, other):
        if type(other)==np.ndarray:
            raise ValueError("You should use @ when multiplying a variable with a numpy array")
        v = CyLPExpr(opr="*", left=self, right=other)
        v.expr = v
        self.expr = v
        return v

    def __rmatmul__(self, other):
        v = CyLPExpr(opr="*", left=other, right=self)
        v.expr = v
        self.expr = v
        return v

    def __matmul__(self, other):
        v = CyLPExpr(opr="*", left=self, right=other)
        v.expr = v
        self.expr = v
        return v
    
    def __rsub__(self, other):
        v = CyLPExpr(opr="-", left=other, right=self)
        v.expr = v
        self.expr = v
        return v

    def __sub__(self, other):
        v = CyLPExpr(opr="-", left=self, right=other)
        v.expr = v
        self.expr = v
        return v

    def __radd__(self, other):
        v = CyLPExpr(opr="+", left=other, right=self)
        v.expr = v
        self.expr = v
        return v

    def __add__(self, other):
        v = CyLPExpr(opr="+", left=self, right=other)
        v.expr = v
        self.expr = v
        return v

    def __neg__(self):
        v = CyLPExpr(opr="u-", right=self)
        v.expr = v
        self.expr = v
        return v

    def getPostfix(self):
        if isinstance(self, CyLPVar):
            return [self]
        left = [self.left]
        right = [self.right]
        if isinstance(self.left, CyLPExpr):
            left = self.left.getPostfix()
        if isinstance(self.right, CyLPExpr):
            right = self.right.getPostfix()

        return left + right + [self.opr]

    def evaluate(self, name=''):
        '''
        Evaluates an expression in the postfix form
        '''
        #FIXME: sometimes calling this twice is different than running it once
        tokens = self.getPostfix()
        operands = []

        cons = CyLPConstraint(name)

        for token in tokens:
            # If an operand found
            if type(token) != str or token not in CyLPExpr.operators:
                operands.append(token)
                if (isinstance(token, CyLPVar)):
                    varToBeAdded = token
                    if not varToBeAdded in cons.variables:
                        cons.variables.append(varToBeAdded)

            # Else we know what to do with an operator
            else:
                if token == 'u-':
                    right = operands.pop()
                    cons.perform(token, right=right)
                    operands.append(CyLPExpr(token, right=right))
                else:
                    right = operands.pop()
                    left = operands.pop()
                    cons.perform(token, left, right)
                    operands.append(CyLPExpr(token, left, right))

        # If something remains in the *operands* list, it means
        # that we have a single CyLPVar object on a line with no
        # operators operating on it. Check this situation and
        # add a 1 vector as its coefficient
        if len(operands) == 1 and isinstance(operands[0], CyLPVar):
            var = operands[0]
            cons.varCoefs[var] = identitySub(var) #np.ones(var.dim)

        # Reset variables for future constraints
        for var in cons.varCoefs.keys():
            var.expr = var

        return cons


class CyLPConstraint:
    gid = 0
    def __init__(self, name=''):
        self.varNames = []
        self.parentVarDims = {}
        self.varCoefs = {}
        self.lower = None
        self.upper = None
        self.nRows = None
        self.isRange = True
        self.variables = []
        if not name:
            CyLPConstraint.gid += 1
            name = 'R_%d' % CyLPConstraint.gid
        self.name = name

    def __repr__(self):
        s = '\n'
        s += 'constraint %s:\n' % self.name
        s += 'variable names:\n'
        s += str(self.varNames) + '\n'
        s += 'coefficients:\n'
        s += str(self.varCoefs) + '\n'
        s += 'lower = %s\n' % str(self.lower)
        s += 'upper = %s\n' % str(self.upper)
        if self.isRange:
            s += 'Constraint is a range\n'
        else:
            s += 'normal Constarint\n'
        return s

    def mul(self, expr, coef):
        '''
        Recursively multiplies all variable coefficients in *expr* by *coef*
        '''
        if isinstance(expr, CyLPVar):
            self.varCoefs[expr] *= coef
            return
        if isinstance(expr, CyLPExpr):
            if expr.left is None:
                return
            self.mul(expr.right, coef)
            self.mul(expr.left, coef)


    def perform(self, opr, left=None, right=None):
        if isinstance(left, (CyLPArray, np.matrix, np.ndarray)):
            left = CyLPArray(np.squeeze(np.asarray(left)))
        if isinstance(right, (CyLPArray, np.matrix, np.ndarray)):
            right = CyLPArray(np.squeeze(np.asarray(right)))

        if isinstance(right, CyLPVar):
            if right.dim == 0:
                return
            if right.name not in self.varNames:
                self.varNames.append(right.name)
                self.parentVarDims[right.name] = right.parentDim
            if opr == 'u-':
                #ones = -CyLPArray(np.ones(right.dim))
                #self.nRows = right.dim
                #self.varCoefs[right] = ones
                self.varCoefs[right] = -identitySub(right)
                self.nRows = len(right.indices)
                self.isRange = False
            elif opr == 'sum':
                n = right.dim
                if right.parent is not None:
                    n = right.parent.dim
                self.varCoefs[right] = sparse.lil_matrix((1, n))
                self.varCoefs[right][0, right.indices] = np.ones(len(right.indices))
                self.nRows = 1
                self.isRange = False
            else:
                #self.varNames.append(right.name)
                #self.parentVarDims[right.name] = right.parentDim
                dim = right.dim

                if opr == "*":  # Setting the coefficients
                    # Ignore a term if its coef is None
                    if isNumber(left):
                        if self.nRows and self.nRows != 1:
                            raise Exception("Expected 1-dimensional" \
                                            " coefficient")
                        #self.nRows = 1
                        #coef = CyLPArray(left * np.ones(dim))
                        coef = left * identitySub(right)
                        self.nRows = len(right.indices)
                    else:
                        if left is None:
                            return
                        if right.dim != left.shape[-1]:
                            raise Exception("Coefficient:\n%s\n has %d" \
                                            " columns expected %d"
                                        % (left, left.shape[1], right.dim))
                        if len(left.shape) == 1:  # Array
                            nr = 1
                        else:
                            nr = left.shape[0]
                            #nr = len(left)
                        if self.nRows and self.nRows != nr:
                            raise Exception("Coefficient:\n%s\n has %d rows" \
                                            " expected %d"
                                        % (left, left.shape[0], self.nRows))
                        self.nRows = nr
                        if right.parent is None:
                            coef = deepcopy(left)
                        else:
                            coef = sparse.lil_matrix((nr, right.parent.dim))
                            coef[:, right.indices] = left

                    self.varCoefs[right] = coef
                    self.isRange = False

                if opr == '+' or opr == '-':
                    # No coefs for left op
                    self.isRange = False
                    if isinstance(left, CyLPVar):
                        self.varCoefs[left] = identitySub(left)
                        self.nRows = len(left.indices)
                        if left.name not in self.varNames:
                            self.varNames.append(left.name)
                            self.parentVarDims[left.name] = left.parentDim

                    # No coefs for right op
                    if isinstance(right, CyLPVar):
                        if right in self.varCoefs.keys():
                            self.varCoefs[right] *= -1
                        else:
                            coef = identitySub(right)
                            if opr == '-':
                                coef *= -1
                            self.varCoefs[right] = coef
                            self.nRows = len(right.indices)
                            #if self.nRows == 0:
                            #    self.nRows = 1
                            #    coef = CyLPArray(np.ones(right.dim))
                            #elif self.nRows == 1:
                            #    coef = CyLPArray(np.ones(1))
                            #else:
                            #    coef = np.matrix(np.eye(self.nRows))

                            #if opr == '-':
                            #    coef *= -1
                            #self.varCoefs[right] = coef
                            if right.name not in self.varNames:
                                self.varNames.append(right.name)
                                self.parentVarDims[right.name] = \
                                                            right.parentDim
        # check if no coef for left
        if left.__class__ == CyLPVar and (opr == '-' or opr == '+'):
            if left.dim == 0 :
                return
            self.isRange = False
            #ones = CyLPArray(np.ones(left.dim))
            #self.varCoefs[left] = ones
            #self.nRows = 1
            self.varCoefs[left] = identitySub(left)
            self.nRows = len(left.indices)
            if left.name not in self.varNames:
                self.varNames.append(left.name)
                self.parentVarDims[left.name] = left.parentDim

        # Coef already set for the right operand
        if right.__class__ == CyLPExpr:
            if opr == '-':
                # The expression on the right is in the form (left opr right)
                # so the key to the coef in self.varCoefs is right.right i.e.
                # the CyLPVar on the right
                #if right.right.dim == 0:
                #    return
                self.isRange = False
                self.mul(right, -1)
                #self.varCoefs[right.right] *= -1
            if opr == '*' and isNumber(left):
                self.mul(right, left)


        if opr in ('<=', '>=', '=='):
            if isinstance(left, CyLPExpr) and not isinstance(right,
                                                              CyLPExpr):
                bound = right
            elif isinstance(right, CyLPExpr) and not isinstance(left,
                                                                CyLPExpr):
                bound = left
            else:
                raise Exception('At least one side of a comparison sign' \
                                'should be constant.')

            # FIXME: check this: I suppose never runs
            if self.isRange:
                var = self.variables[0]
                self.nRows = var.dim
                dim = var.dim
            #if not self.nRows:
                #self.nRows = left.parentDim
                #dim = left.parentDim
            else:
#                # FIXME: Think of the correct way to check bound dimensions
#                if (not isinstance(right, (float, long, int)) and
#                            right.shape[0] != self.nRows):
#                    #raise Exception('Bound dim: %d, expected dim: %d '
#                    #                % (right.shape[0], self.nRows))
#                    pass
                dim = self.nRows
#
            if isNumber(right):
                bound = CyLPArray(right * np.ones(dim))

            if self.isRange:
                if ((opr in ('>=', '==') and isinstance(left, CyLPExpr)) or
                    (opr in ('<=', '==') and isinstance(right, CyLPExpr))):
                    if var.parent:
                        var.parent.lower[var.indices] = bound
                    else:
                        var.lower[var.indices] = bound

                if ((opr in ('<=', '==') and isinstance(left, CyLPExpr)) or
                    (opr in ('>=', '==') and isinstance(right, CyLPExpr))):
                    if var.parent:
                        var.parent.upper[var.indices] = bound
                    else:
                        var.upper[var.indices] = bound

            else:
                if ((opr in ('>=', '==') and isinstance(left, CyLPExpr)) or
                    (opr in ('<=', '==') and isinstance(right, CyLPExpr))):
                    self.lower = bound
                    if self.upper is None:
                        self.upper = getCoinInfinity() * np.ones(len(bound))
                if ((opr in ('<=', '==') and isinstance(left, CyLPExpr)) or
                    (opr in ('>=', '==') and isinstance(right, CyLPExpr))):
                    self.upper = bound
                    if self.lower is None:
                        self.lower = -getCoinInfinity() * np.ones(len(bound))


class CyLPVar(CyLPExpr):
    '''
    Contains variable information such as its name, dimension,
    bounds, and whether or not it is an integer. We don't creat
    instances directly but rather use :py:func:`CyLPModel.addVariable`
    to create one.

    See the :ref:`modeling example <modeling-usage>`.

    '''

    def __init__(self, name, dim, isInt=False, fromInd=None, toInd=None):
        self.name = name
        self.dims = None
        if isinstance(dim, tuple):
            self.setDims(dim)
            self.dim = dim = reduce(mul, dim)
            self.parentDim = dim
        else:
            self.dim = dim
            self.parentDim = dim
        self.parent = None
        self.isInt = isInt
        self.expr = self
        self.lower = -getCoinInfinity() * np.ones(dim)
        self.upper = getCoinInfinity() * np.ones(dim)

        self.indices = np.arange(dim, dtype=np.int32)
        if fromInd and toInd:
            #self.indices = np.arange(fromInd, toInd, dtype=np.int32)
            self.fromInd = fromInd
            self.toInd = toInd
        else:
            self.fromInd = 0
            self.toInd = dim-1

    def __repr__(self):
        s = self.name
        if self.fromInd and self.toInd:
            s += '[%d:%d]' % (self.fromInd, self.toInd)
        elif self.parent is not None and len(self.indices) == 1:
            s += '[%d]' % self.indices
        return s

    def setDims(self, ds):
        self.dims = ds

    def __getitem__(self, key):
        if type(key) == int:
            newObj = CyLPVar(self.name, 1)
            newObj.indices = np.array([key], np.int32)
            newObj.parentDim = self.dim
            newObj.parent = self
            newObj.dim = 1
        elif type(key) == slice:
            newObj_fromInd = key.start
            if key.stop > self.dim:
                newObj_toInd = self.dim
            else:
                newObj_toInd = key.stop

            newObj = CyLPVar(self.name, newObj_toInd - newObj_fromInd)
            newObj.parentDim = self.dim
            newObj.parent = self
            newObj.fromInd = newObj_fromInd
            newObj.toInd = newObj_toInd
            newObj.indices = np.arange(newObj.fromInd, newObj.toInd,
                                       dtype='int32')
            # Save parentDim for future use
            newObj.dim = len(newObj.indices)
        elif isinstance(key, (np.ndarray, list)):
            newObj = CyLPVar(self.name, len(key))
            newObj.parentDim = self.dim
            newObj.parent = self
            newObj.fromInd = None
            newObj.toInd = None
            newObj.indices = np.array(key)
            newObj.dim = len(newObj.indices)

        elif isinstance(key, tuple):
            inds = []
            n = range(len(key))
            for i in n:
                k = key[i]
                if isinstance(k, int):
                    inds.append(Ind(slice(k, k+1), self.dims[i]))
                else:
                    inds.append(Ind(k, self.dims[i]))
            newObj = self.__getitem__(getMultiDimMatrixIndex(inds))
        return newObj

    def sum(self):
        v = CyLPExpr(opr="sum", right=self)
        v.expr = v
        self.expr = v
        return v


class CyLPArray(np.ndarray):
    '''
    It is a tweaked Numpy array. It allows user to define objects
    comparability to an array. If we have an instance of ``CyLPVar``
    called ``x`` and a numpy array ``b``, then ``b >= x`` returns an array
    of the same dimension as ``b`` with all elements equal to ``False``,
    which has no sense at all. On the constrary ``CyLPArray`` will return
    ``NotImplemented`` if it doesn't know how to compare. This is particularly
    helpful when we define LP constraints.

    See the :ref:`modeling example <modeling-usage>`.

    '''

    def __new__(cls, input_array, info=None):
        # Input array is an already formed ndarray instance
        # We first cast to be our class type
        obj = np.asarray(input_array).view(cls)
        # add the new attribute to the created instance
        obj.__le__ = CyLPArray.__le__
        obj.__ge__ = CyLPArray.__ge__
        return obj

    def __le__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__le__(self, other)

    def __ge__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__ge__(self, other)

    def __mul__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__mul__(self, other)

    def __rmul__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__rmul__(self, other)

    def __add__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__add__(self, other)

    def __radd__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__radd__(self, other)

    def __rsub__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__rsub__(self, other)

    def __sub__(self, other):
        if (issubclass(other.__class__, CyLPExpr)):
            return NotImplemented
        return np.ndarray.__sub__(self, other)


class IndexFactory:
    '''
    A small class that keeps track of the indices of variables and constraints
    that you are adding to a problem gradually.

    **Usage**

    >>> import numpy as np
    >>> from cylp.py.modeling.CyLPModel import IndexFactory
    >>> i = IndexFactory()
    >>> i.addVar('x', 10)
    >>> i.addVar('y', 5)
    >>> i.hasVar('x')
    True
    >>> (i.varIndex['y'] == np.arange(10, 15)).all()
    True
    >>> i.addConst('Manpower', 4)
    >>> i.addConst('Gold', 10)
    >>> (i.constIndex['Manpower'] == np.arange(4)).all()
    True
    >>> (i.constIndex['Gold'] == np.arange(4, 14)).all()
    True

    '''

    def __init__(self):
        self.varIndex = {}
        self.constIndex = {}
        self.currentVarIndex = 0
        self.currentConstIndex = 0

    def addVar(self, varName, numberOfVars):
        if numberOfVars == 0:
            return
        if not varName:
            raise Exception('You must specify a name for a variable.')
        if varName in self.varIndex.keys():
            print('Variable already exists.')
            #self.varIndex[varName] += range(self.currentVarIndex,
            #                                self.currentVarIndex +
            #                                numberOfVars)
        else:
            self.varIndex[varName] = np.arange(self.currentVarIndex,
                                            self.currentVarIndex +
                                            numberOfVars, dtype='int32')
        self.currentVarIndex += numberOfVars

    def removeVar(self, name):
        if not self.hasVar(name):
            raise Exception('Variable "%s" does not exist.' % name)
        nVars = len(self.varIndex[name])
        start = self.varIndex[name][0]
        del self.varIndex[name]
        for varName in self.varIndex.keys():
            inds = self.varIndex[varName]
            if inds[0] > start:
                self.varIndex[varName] = inds - nVars * np.ones(len(inds),
                                                                np.int32)

    def hasVar(self, varName):
        return varName in self.varIndex.keys()

    def hasConst(self, constName):
        return constName in self.constIndex

    def getLastVarIndex(self):
        return self.currentVarIndex - 1

    def addConst(self, constName, numberOfConsts):
        if numberOfConsts == 0:
            return
        if not constName:
            raise Exception('You must specify a name for a constraint.')
        if self.hasConst(constName):
            print('Constraint already exists: %s' % constName)
            #self.constIndex[constName] += range(self.currentConstIndex,
            #        self.currentConstIndex + numberOfConsts)
        else:
            self.constIndex[constName] = np.arange(self.currentConstIndex,
                                            self.currentConstIndex +
                                            numberOfConsts, dtype='int32')
        self.currentConstIndex += numberOfConsts

    def removeConst(self, name):
        if not self.hasConst(name):
            raise Exception('Constraint "%s" does not exist.' % name)
        nCons = len(self.constIndex[name])
        start = self.constIndex[name][0]
        del self.constIndex[name]
        for constName in self.constIndex.keys():
            inds = self.constIndex[constName]
            if inds[0] > start:
                self.constIndex[constName] = inds - nCons * np.ones(len(inds),
                                                                    np.int32)
        self.currentConstIndex -= nCons


    def getLastConstIndex(self):
        return self.currentConstIndex - 1

    def __repr__(self):
        varind = self.varIndex
        cind = self.constIndex

        s = "variables : \n"
        for vname, rg in varind.items():
            s += '%s : %s\n' % (vname.rjust(15), str(rg))
        s += '\n'
        s += "constraints : \n"
        for cname, rg in cind.items():
            s += '%s : %s\n' % (cname.rjust(15), str(rg))
        return s

    def reverseVarSearch(self, ind):
        '''
        Take an index and return the corresponding variable name.
        '''
        inds = self.varIndex
        for key in inds.keys():
            if ind in inds[key]:
                i = np.where(inds[key]==ind)[0][0]
                return key, i
        return -1, -1


from cylp.py.utils.sparseUtil import sparseConcat, csr_matrixPlus, csc_matrixPlus

class CyLPModel(object):
    '''
    Hold all the necessary information to create a linear program, i.e.
    variables, constraints, and the objective function.

    See the :ref:`modeling example <modeling-usage>`.

    '''

    def __init__(self):
        self.variables = []
        self.constraints = []
        self.objective_ = None
        self.inds = IndexFactory()
        self.nVars = 0
        self.varNames = []
        self.nCons = 0
        self.pvdims = {}

    def addVariable(self, name, dim, isInt=False):
        '''
        Create a new instance of :py:class:`CyLPVar` using the given
        arguments and add it to current model's variable list.
        '''

        if dim == 0:
            return

        #If mulidim, correct dim
        if isinstance(dim, tuple):
            numVars = reduce(mul, dim)
        else:
            numVars = dim
        var = CyLPVar(name, dim, isInt, self.nVars, self.nVars+numVars)
        self.variables.append(var)

        if not self.inds.hasVar(var.name):
            self.inds.addVar(var.name, numVars)
            self.nVars += numVars
            self.varNames.append(var.name)
            self.pvdims[var.name] = numVars

            if var.dims:
                var.mpsNames = [var.name + '_' + '_'.join(x) for x in \
                        product(*[map(str, range(i)) for i in var.dims])]
            else:
                var.mpsNames = ['%s_%s' % (var.name, i) for i in range(var.dim)]

            o = self.objective_
            if isinstance(o, np.ndarray):
                o = np.concatenate((o, np.zeros(numVars)), axis=0)
            else:
                o = sparseConcat(o, csr_matrixPlus(np.zeros(numVars)), 'h')

            # I'm not exactly sure why the objective gets changed into
            # csr_matrixPlus here when it may not be coming in. Shouldn't it
            # just always be?
            self.objective_ = csr_matrixPlus(o)

        else:
            raise Exception('Varaible %s already exists.' % var.name)

        return var

    def removeVariable(self, name):
        '''
        Remove a variable named ``name`` from the model
        '''
        if not self.inds.hasVar(name):
            raise Exception('Variable "%s" does not exist.' % name)

        self.nVars -= self.pvdims[name]
        start = self.inds.varIndex[name][0]
        end = start + self.pvdims[name]
        o = self.objective_

        if isinstance(o, np.ndarray):
            o = np.concatenate((o[:start], o[end:]), axis=0)
        else:
            if end == o.shape[1]:
                if start == 0:
                    print('Problem empty.')
                else:
                    o = o[0, :start]
            elif start == 0:
                o = o[0, end:]
            else:
                o = sparseConcat(o[0, :start], o[0, end:], how='h')

        # I'm not exactly sure why the objective gets changed into
        # csr_matrixPlus here when it may not be coming in. Shouldn't it
        # just always be?
        self.objective_ = csr_matrixPlus(o)

        del self.pvdims[name]
        self.varNames.remove(name)
        self.inds.removeVar(name)
        for i in range(len(self.variables)):
            var = self.variables[i]
            if var.name == name:
                del self.variables[i]
                break

        #Removing the variable from the constraints
        cons = self.constraints
        for c in cons:
            if name in c.varNames:
                c.varNames.remove(name)
                del c.parentVarDims[name]
                for v in list(c.varCoefs.keys()):
                    if v.name == name:
                        del c.varCoefs[v]

        for c in cons[:]:
            self.inds
            if not c.varCoefs:
                self.removeConstraint(c.name)

    def getVarByName(self, varName):
        '''
        Return a variable with name ``varName``.
        '''
        for var in self.variables:
            if var.name == varName:
                return var
        return None

    def makeIndexFactory(self):
        self.allVarNames = []
        varNameDict = {} #copy of allVarNames in dictionary form, for efficiency
        self.allParentVarDims = {}
        self.nRows = 0
        for c in self.constraints:
            # Make sure to add variables in the same order they were
            # added to CyLPModel
            for var in self.variables:
                for vname in c.varNames:
                    if vname == var.name and vname not in varNameDict:
                        self.allVarNames += [vname]
                        varNameDict[vname] = True
            self.allParentVarDims.update(c.parentVarDims)
            self.nRows += c.nRows

    @property
    def objective(self):
        return self.objective_

    @objective.setter
    def objective(self, obj):
        if isinstance(obj, CyLPExpr):
            self.objective_ = obj.evaluate()
            obj = None
            #for varName in self.allVarNames:
            for varName in self.varNames:
                v_coef = self.generateVarObjCoef(varName)
                obj = sparseConcat(obj, v_coef, how='h')
            obj = np.squeeze(np.asarray(obj.todense()))
        self.objective_ = obj

    def evaluateAt(self, x0):
        return np.dot(self.objective, x0)

    def __iadd__(self, cons):
        '''
        Call :meth:`addConstraint`. The only difference is that
        you cannot specify a name for your constraint.
        '''
        self.addConstraint(cons)
        return self

    def addConstraint(self, cons, consName='', addMpsNames=True):
        '''
        Add constraint ``cons`` to the ``CyLPModel``. Argument ``cons`` must
        be an expresion made using instances of :py:class:`CyLPVar`,
        :py:class:`CyLPArray`, Numpy matrices, or sparse matrices. It can
        use normal operators such as ``>=``, ``<=``, ``==``, ``+``, ``-``,
        ``*``.

        See the :ref:`modeling example <modeling-usage>`.

        '''

        c = cons.evaluate(consName)
        if not c.isRange:
            self.constraints.append(c)
            if c.name:
                self.inds.addConst(c.name, c.nRows)
            self.nCons += c.nRows

        if addMpsNames:
            c.mpsNames = []
            for i in range(c.nRows):
                c.mpsNames.append('%s_%s' % (c.name, str(i)))


        #self.makeMatrices()
        return c

    def removeConstraint(self, name):
        '''
        Remove constraint called ``name`` and return its CLP indices back
        for actual removal.
        '''
        if not self.inds.hasConst(name):
            raise Exception('Constraint "%s" does not exist.' % name)

        for i in range(len(self.constraints)):
            if self.constraints[i].name == name:
                con = self.constraints[i]
                break
        self.nCons -= con.nRows
        del self.constraints[i]
        indsOfConstriantsToBeRemoved = self.inds.constIndex[name]
        self.inds.removeConst(name)
        return indsOfConstriantsToBeRemoved

    def generateVarObjCoef(self, varName):
        #dim = self.allParentVarDims[varName]
        dim = self.pvdims[varName]
        coef = csr_matrixPlus((1, dim))
        obj = self.objective_
        keys = [k for k in obj.varCoefs.keys() if k.name == varName]

        for var in keys:
            coef = coef + obj.varCoefs[var]
        return coef

    def generateVarMatrix(self, varName):
        '''
        Create the coefficient matrix for a variable named
        varName. Return a csr_matrixPlus.
        '''
        #dim = self.allParentVarDims[varName]
        dim = self.pvdims[varName]
        mainCoef = None
        for c in self.constraints:
            coef = sparse.coo_matrix((c.nRows, dim))
            keys = [k for k in c.varCoefs.keys() if k.name == varName]
            for var in keys:
                coef = coef + c.varCoefs[var]
            mainCoef = sparseConcat(mainCoef, coef, 'v')

        return mainCoef

    def makeMatrices(self):
        '''
        Makes coef matrix and rhs vector from CyLPConstraints
        in self.constraints
        '''
        #if len(self.constraints) == 0:
        #    return
        self.makeIndexFactory()
        #self.getVarBounds()


        # Create the aggregated coef matrix
        masterCoefMat = None

        masterCoefMat = None
        if self.nCons > 0:
            for varName in self.varNames:# self.pvdims.keys():#self.allVarNames:
                vmat = self.generateVarMatrix(varName)

                if vmat is None:
                    vmat = csc_matrixPlus((self.nCons, self.pvdims[varName]))
                masterCoefMat = sparseConcat(masterCoefMat, vmat, 'h')

        # Create bound vectors
        c_lower = np.array([])
        c_upper = np.array([])
        for c in self.constraints:
            if not c.isRange:
                c_lower = np.concatenate((c_lower, c.lower), axis=0)
                c_upper = np.concatenate((c_upper, c.upper), axis=0)
        # Create variables bound vectors
        v_lower = np.array([])
        v_upper = np.array([])
        if self.nVars:
            v_lower = -getCoinInfinity() * np.ones(self.nVars)
            v_upper = getCoinInfinity() * np.ones(self.nVars)
            for v in self.variables:
                varinds = self.inds.varIndex[v.name]
                v_lower[varinds] = v.lower
                v_upper[varinds] = v.upper

                #v_lower = np.concatenate((v_lower, v.lower), axis=0)
                #v_upper = np.concatenate((v_upper, v.upper), axis=0)
        #if masterCoefMat is not None:
        return masterCoefMat, c_lower, c_upper, v_lower, v_upper


class CyLPSolution:

    def __init__(self):
        self.sol = {}

    def add(self, key, val):
        'Add (key, val) to self.dic. Ignore if val is zero'
        if val == 0: # See if necessary to use a tolerance
            return
        self.sol[key] = val

    def getVal(self, key):
        'Return the value corresponing key'
        if key not in self.sol.keys():
            return 0
        return self.sol[key]

    def __getitem__(self, key):
        #if isinstance(key, tuple):
        if key in self.sol.keys():
            return self.sol[key]
        else:
            return 0

    def __setitem__(self, key, val):
        #if isinstance(key, tuple):
        if val == 0: # See if necessary to use a tolerance
            return
        self.sol[key] = val
    def __repr__(self):
        return repr(self.sol)

def getCoinInfinity():
    return 1.79769313486e+308


if __name__ == '__main__':
    from cylp.cy import CyClpSimplex
    from cylp.py.modeling.CyLPModel import CyLPArray
    s = CyClpSimplex()
    x = s.addVariable('x', (5, 3, 6))
    s += 2 * x[2, :, 3].sum() + 3 * x[0, 1, :].sum() >= 5

    s += 0 <= x <= 1
    c = CyLPArray(range(18))

    s.objective = c * x[2, :, :] + c * x[0, :, :]
    s.writeMps('/Users/mehdi/Desktop/test.mps')
    s.primal()
    sol = s.primalVariableSolution
    print(sol)

#model = CyLPModel()
#
#x = model.addVariable('x', 5)
#y = model.addVariable('y', 4)
#z = model.addVariable('z', 5)
#
#
#b = CyLPArray([3.1, 4.2])
#aa = np.matrix([[1, 2, 3, 3, 4], [3, 2, 1, 2, 5]])
#aa = np.matrix([[0, 0, 0, -1.5, -1.5], [-3, 0, 0, -2,0 ]])
#dd = np.matrix([[1, 3, 1, 4], [2, 1,3, 5]])
#
#
#model.addConstraint(1.5 <=  x[1:3] + 5 * y[3:] - 6 * x[2:5]  + x[0] <= 14)
#
#print(model.constraints[0].varCoefs)
#
#model.addConstraint(2 <= aa*x + dd * y <= b)
#model.addConstraint(x[2:4] >= 3)
#model.addConstraint(1.1 <= y <= CyLPArray([1,2,3,4]))
#
#
#A = model.makeMatrices()
#
#print('x:')
#print(x.lower)
#print(x.upper)
#print('y:')
#print(y.lower)
#print(y.upper)
#
#print('+++++++++++++++++++++++++++++')
##cc = x[0] - 2*x[1:3] -2.5 * y[1]
##cc = cc.evaluate()
#model.objective = x[0] - 2*x[1:3] -2.5 * y[1]
##print(model.allParentVarDims)
##print(model.generateVarObjCoef('x'))
##print(model.generateVarObjCoef('y'))
#
#print(model.objective)
##print(cl)
##print(cu)



'''
This class defines an interface all the Python DUAL pivot rules must implement.
It consists of a method :py:func:`pivotRow` which must be implemented.
'''


class DualPivotPythonBase:
    def pivotRow(self):
        '''
        Every subclass of ``DualPivotPythonBase``, i.e. every pivot rule, must
        implement this method.

        Return the index of a selected row (constraint), an integer.
        '''
        raise NotImplementedError('pivotRow is not implemented')





cimport numpy as np

cdef extern from "ICbcNode.hpp":
    cdef cppclass CppICbcNode "ICbcNode":
        int depth()
        int numberUnsatisfied()
        double sumInfeasibilities()
        bint active()
        bint onTree()
        double objectiveValue()
        bint breakTie(CppICbcNode* y)

cdef class CyCbcNode:
    cdef CppICbcNode* CppSelf
    cdef setCppSelf(self, CppICbcNode* cbcnode)



#!/usr/bin/env python

# Usage:
# The command:
# > python createCythonInterface.py Person
# creates the appropriate Cython pyx and pxd files


import sys


className = sys.argv[1]
cyclassname = 'Cy' + className
pyxFile = cyclassname + '.pyx'
pxdFile = cyclassname + '.pxd'

# First create the pxd file:
pxdContent = \
'''
cdef extern from "%s.hpp":
    cdef cppclass Cpp%s "%s":
        pass
    Cpp%s *new_Cpp%s "new %s" ()

cdef class Cy%s:
    cdef Cpp%s* CppSelf
    cdef setCppSelf(self, Cpp%s* s)
''' % (className, className, className, className, className,
            className, className, className, className)

with open(pxdFile, 'w') as f:
    f.write(pxdContent)


#Create the pyx file
pyxContent = '''

cdef class Cy%s:
    'Cy%s documentation'
    def __cinit__(self):
        self.CppSelf = new Cpp%s()

    cdef setCppSelf(self, Cpp%s* s):
        del self.CppSelf
        self.CppSelf = s

''' % (className, className, className, className, className)

with open(pyxFile, 'w') as f:
    f.write(pyxContent)





from __future__ import print_function
import numpy as np
from scipy import sparse
from cylp.cy import CyClpSimplex
from cylp.py.utils.sparseUtil import csr_matrixPlus
from cylp.py.modeling.CyLPModel import CyLPArray
from cylp.py.QP.QP import QP

class setCover:
    def __init__(self):
        self.nRows = self.nCols = 0
        self.cols = []
        self.costs = []

    def readWedelin(self, filename):
        '''
        Read a file describing a set-covering problem:
        #rows #ncols
        for each col:
            cost, #1's, list of 1 rows

        '''
        with open(filename, 'r') as f:
            lines = f.read().splitlines()
            tokens = lines[0].split()
            self.nRows = int(tokens[0])
            self.nCols = int(tokens[1])
            self.cols = []
            self.costs = []
            for line in lines[1:]:
                tokens = line.split()
                self.cols.append([int(i)-1 for i in tokens[2:]])
                self.costs.append(float(tokens[0]))

        print(self.nRows, self.nCols)

    def readBalas(self, filename):
        '''
        Read a file describing a set-covering problem:
        #ncols #nRows
        column costs
        for each col:
            #1's, list of 1 rows

        '''
        with open(filename, 'r') as f:
            lines = f.read().splitlines()
            tokens = lines[0].split()
            self.nRows = int(tokens[1])
            self.nCols = int(tokens[0])

            i = 1
            self.costs = []

            while len(self.costs) != self.nCols:
                self.costs += [float(k) for k in lines[i].split()]
                i += 1


            self.cols = []
            j = i
            while len(self.cols) != self.nCols:
                self.cols.append([])
                tokens = lines[j].split()
                nnz = int(tokens[0])
                tokens = tokens[1:]
                self.cols[-1] += [int(i)-1 for i in tokens]
                j += 1
                while len(self.cols[-1]) != nnz:
                    self.cols[-1] += [int(i)-1 for i in lines[j].split()]
                    j += 1
        print(self.nRows, self.nCols)


    @property
    def model(self):
        A = self.A
        c = self.c
        s = CyClpSimplex()

        x = s.addVariable('x', self.nCols)

        s += A * x >= 1
        s += 0 <= x <= 1

        s.objective = c * x

        return s


    def QPModel(self, addW=False):
        A = self.A
        c = self.c
        s = CyClpSimplex()

        x = s.addVariable('x', self.nCols)
        if addW:
            w = s.addVariable('w', self.nCols)

        s += A * x >= 1
        n = self.nCols

        if not addW:
            s += 0 <= x <= 1
        else:
            s += x + w == 1
            s += 0 <= w <= 1

##        s += -1 <= x <= 1

        s.objective = c * x

        if addW:
            G = sparse.lil_matrix((2*n, 2*n))
            for i in range(n/2, n): #range(n-1):
                G[i, i] = 1
            G[2*n-1, 2*n-1] = 10**-10
        else:
            G = sparse.lil_matrix((n, n))
            for i in range(n/2, n): #range(n-1):
                G[i, i] = 1


        s.Hessian = G
        return s

    @property
    def A(self):
        a = sparse.lil_matrix((self.nRows, self.nCols))
        for nCol in range(self.nCols):
            for nRow in self.cols[nCol]:
                a[nRow, nCol] = 1

        return csr_matrixPlus(a)

    @property
    def c(self):
        return CyLPArray(self.costs)
##        c = np.empty((self.nCols,), np.double)
##        print(self.nRows, self.nCols)
##        for nCol in range(self.nCols):
##            c[nCol] = self.costs[nCol]
##
##        return c

    @property
    def G(self):
        n = self.nCols

##        qp = QP()
##        qp.fromQps('/Users/mehdi/Documents/work/benchmarks/qp/CVXQP3_M.SIF')
##        #mm = qp.G[:n, :n].todense()
##        #print((mm == mm.T).all())
##        #l = np.linalg.eigvals(mm)
##        #print(min(l))
##        G = sparse.lil_matrix((2*n, 2*n))
####        G[n/2:n, n/2:n] = qp.G[n/2:n, n/2:n]
####        k = 10
####        G[n-k:n, n-k:n] = qp.G[:k, :k]
##        dim = min(qp.G.shape[0], n)
##        G[:dim, :dim] = qp.G[:dim, :dim]
##        if G[2*n-1, 2*n-1] == 0:
##            G[2*n-1, 2*n-1] = 10**-10
##        return G

        #n *= 2
        G = sparse.lil_matrix((2*n, 2*n))
        for i in range(n/2, n): #range(n-1):
            G[i, i] = 1
            #G[i+1, i] = -0.2
            #G[i, i+1] = -0.2
        #G[n - 1, n - 1] = 1
        G[2*n-1, 2*n-1] = 10**-10
        return csr_matrixPlus(G)


import sys




s = setCover()
#s.readWedelin(sys.argv[1])
s.readBalas(sys.argv[1])

of = sys.argv[2]
##f = filename[(filename.rindex('/') + 1): (filename.rindex('.'))] + '_X.qps'
##m = s.QPModel(False)
##m.writeMps('/Users/mehdi/temp/scp/' + f)
#f = filename[(filename.rindex('/') + 1): (filename.rindex('.'))] + '_W+.qps'

m = s.model
m.writeMps(of)


#m.primal()
#sol = m.primalVariableSolution['x']
#print([s.cols[i] for i in range(s.nCols) if sol[i] == 1])



class NodeCompareBase:
    '''
    Abstract base class
    '''
    def compare(self, x, y):
        '''
        **Abstract Method**

        Return True if ``y`` is better than ``x``
        :arg x: A B&B node
        :type x: CyCbcNode
        :arg y: A B&B node
        :type y: CyCbcNode
        :rtype: bool
        '''
        raise NotImplementedError('SimpleNodeCompare object must ' \
                        'implement *compare*')

    def newSolution(self, model,
                            objectiveAtContinuous,
                            numberInfeasibilitiesAtContinuous):
        '''
        **Abstract Method**

        This method is called after a solution is found
        in a node.
        :rtype: None
        '''
        raise NotImplementedError('SimpleNodeCompare object must ' \
                        'implement *newSolution*')

    def every1000Nodes(self, model, numberNodes):
        '''
        **Abstract Method**

        This method is called each 1000 nodes for possible
        Changes in strategy. Return True to ask for a tree re-sort.
        :rtype: bool
        '''
        raise NotImplementedError('SimpleNodeCompare object must ' \
                        'implement *every1000Nodes*')



from cython.operator cimport dereference as deref
cimport cpython.ref as cpy_ref
from cpython.ref cimport PyObject
from cylp.cy cimport CyClpSimplex
from cylp.cy.CyCoinIndexedVector cimport CppCoinIndexedVector
import numpy as np
cimport numpy as np

cdef extern from "ClpDualRowPivot.hpp":
    cdef cppclass CyClpDualRowPivot "ClpDualRowPivot":
        pass
    CyClpDualRowPivot* new_CyClpDualRowPivot \
                                    "new ClpDualRowPivot" ()

cdef extern from "ClpFactorization.hpp":
    cdef cppclass ClpFactorization:
        int updateColumnTranspose (CppCoinIndexedVector * regionSparse,
                  CppCoinIndexedVector * regionSparse2)

cdef extern from "IClpDualRowPivotBase.h":
    #cdef cppclass CoinIndexedVector:
    #   pass
    ctypedef int (*runPivotRow_t)(void *obj)
    ctypedef CyClpDualRowPivot* (*runDualPivotClone_t)(void *obj, bint copyData)
    ctypedef double (*runUpdateWeights_t)(void* obj,
                                  CppCoinIndexedVector* inp,
                                  CppCoinIndexedVector* spare,
                                  CppCoinIndexedVector* spare2,
                                  CppCoinIndexedVector* updatedColumn)

    ctypedef void (*runUpdatePrimalSolution_t)(void* obj,
                                       CppCoinIndexedVector * inp,
                                       double theta,
                                       double * changeInObjective)

    cdef cppclass CppClpDualRowPivotBase:
        CyClpSimplex.CppIClpSimplex* model()
        void setModel(CyClpSimplex.CppIClpSimplex* m)
        CppClpDualRowPivotBase(PyObject* obj,
                                    runPivotRow_t runPivotRow,
                                    runDualPivotClone_t runDualPivotClone,
                                    runUpdateWeights_t runUpdateWeights,
                                    runUpdatePrimalSolution_t runUpdatePrimalSolution)

        int pivotRow()

        CyClpDualRowPivot* clone(bint copyData)
        double updateWeights(CppCoinIndexedVector* inp,
                                  CppCoinIndexedVector* spare,
                                  CppCoinIndexedVector* spare2,
                                  CppCoinIndexedVector* updatedColumn)

        void updatePrimalSolution(CppCoinIndexedVector * inp,
                                       double theta,
                                       double * changeInObjective)



cdef int RunPivotRow(void *ptr)

cdef CyClpDualRowPivot* RunDualPivotClone(void *ptr, bint copyData)
cdef double RunUpdateWeights(void *ptr, CppCoinIndexedVector* inp,
                                  CppCoinIndexedVector* spare,
                                  CppCoinIndexedVector* spare2,
                                  CppCoinIndexedVector* updatedColumn)
cdef void RunUpdatePrimalSolution(void *ptr,
                                       CppCoinIndexedVector * inp,
                                       double theta,
                                       double * changeInObjective)


cdef class CyClpDualRowPivotBase:
    cdef CppClpDualRowPivotBase* CppSelf
    cdef CyClpSimplex.CyClpSimplex cyModel
    #cpdef cyModel
    cdef pivotRow(self)

    cdef CyClpDualRowPivot * clone(self, bint copyData)
    cdef double updateWeights(self, CppCoinIndexedVector* inp,
                                  CppCoinIndexedVector* spare,
                                  CppCoinIndexedVector* spare2,
                                  CppCoinIndexedVector* updatedColumn)
    cdef void updatePrimalSolution(self, CppCoinIndexedVector * inp,
                                       double theta,
                                       np.ndarray[np.double_t,ndim=1] changeInObjective)
    cdef CyClpSimplex.CppIClpSimplex* model(self)
    cdef void setModel(self, CyClpSimplex.CppIClpSimplex* m)
    cdef double* getReducedCosts(self)



from .CyCoinIndexedVector import CyCoinIndexedVector
from .CyClpSimplex import CyClpSimplex
from .CyCbcNode import CyCbcNode
from .CyClpPrimalColumnPivotBase import CyClpPrimalColumnPivotBase
from .CyCoinMpsIO import CyCoinMpsIO
from .CyCoinPackedMatrix import CyCoinPackedMatrix
from .CyCbcModel import CyCbcModel
from .CyCoinModel import CyCoinModel
#
from .CyPivotPythonBase import CyPivotPythonBase
from .CyDantzigPivot import CyDantzigPivot
from .CyPEPivot import CyPEPivot
from .CyWolfePivot import CyWolfePivot



'''
As a part of ``cylp.python.pivots`` it implements Dantzig's
Classical Simplex pivot rule. Although it already exists in CLP,
for testing purposes we implement one in Python.
'''

import sys
import numpy as np
from operator import itemgetter
from random import shuffle
from math import floor
from .PivotPythonBase import PivotPythonBase


class DantzigPivot(PivotPythonBase):
    '''
    Dantzig's pivot rule implementation.

    **Usage**

    >>> from cylp.cy import CyClpSimplex
    >>> from cylp.py.pivots import DantzigPivot
    >>> from cylp.py.pivots.DantzigPivot import getMpsExample
    >>> # Get the path to a sample mps file
    >>> f = getMpsExample()
    >>> s = CyClpSimplex()
    >>> s.readMps(f)  # Returns 0 if OK
    0
    >>> pivot = DantzigPivot(s)
    >>> s.setPivotMethod(pivot)
    >>> s.primal()
    'optimal'
    >>> round(s.objectiveValue, 5)
    2520.57174

    '''

    def __init__(self, clpModel):
        self.dim = clpModel.nRows + clpModel.nCols
        self.clpModel = clpModel

    def pivotColumn(self, updates, spareRow1, spareRow2, spareCol1, spareCol2):
        'Finds the variable with the best reduced cost and returns its index'
        s = self.clpModel

        # Update the reduced costs, for both the original and the slack variables
        if updates.nElements:
            s.updateColumnTranspose(spareRow2, updates)
            s.transposeTimes(-1, updates, spareCol2, spareCol1)
            s.reducedCosts[s.nVariables:][updates.indices] -= updates.elements[:updates.nElements]
            s.reducedCosts[:s.nVariables][spareCol1.indices] -= spareCol1.elements[:spareCol1.nElements]
        updates.clear()
        spareCol1.clear()

        rc = s.reducedCosts
        tol = s.dualTolerance

        indicesToConsider = np.where(s.varNotFlagged & s.varNotFixed &
                                     s.varNotBasic &
                                     (((rc > tol) & s.varIsAtUpperBound) |
                                     ((rc < -tol) & s.varIsAtLowerBound) |
                                     s.varIsFree))[0]

        #freeVarInds = np.where(s.varIsFree)
        #rc[freeVarInds] *= 10

        rc2 = np.abs(rc[indicesToConsider])

        checkFree = True
        #rc2[np.where((status & 7 == 4) | (status & 7 == 0))] *= 10
        if rc2.shape[0] > 0:
            if checkFree:
                w = np.where(s.varIsFree)[0]
                if w.shape[0] > 0:
                    ind = s.argWeightedMax(rc2, indicesToConsider, 1, w)
                else:
                    ind = np.argmax(rc2)
            else:
                    ind = np.argmax(rc2)
            #del rc2
            return  indicesToConsider[ind]
        return -1

    def saveWeights(self, model, mode):
        self.clpModel = model

    def isPivotAcceptable(self):
        return True


def getMpsExample():
    import os
    import inspect
    cylpDir = os.environ['CYLP_SOURCE_DIR']
    return os.path.join(cylpDir, 'cylp', 'input', 'p0033.mps')


if __name__ == "__main__":
    if len(sys.argv) == 1:
        import doctest
        doctest.testmod()
    else:
        from cylp.cy import CyClpSimplex
        from cylp.py.pivots import DantzigPivot
        s = CyClpSimplex()
        s.readMps(sys.argv[1])
        pivot = DantzigPivot(s)
        s.setPivotMethod(pivot)
        s.primal()



#UNDER DEVELOPMENT AND TEST

from __future__ import print_function
import sys
import cProfile
import inspect
from time import perf_counter
import numpy as np
from scipy import sparse
from cylp.cy import CyClpSimplex
from cylp.py.QP.QPSReader import readQPS
from cylp.py.pivots.WolfePivot import WolfePivot
from cylp.py.pivots.WolfePivotPE import WolfePivotPE
from cylp.py.pivots.PositiveEdgeWolfePivot import PositiveEdgeWolfePivot
from cylp.cy import CyCoinModel
from cylp.py.utils.sparseUtil import I, sparseConcat
from cylp.py.modeling.CyLPModel import CyLPModel, CyLPArray, CyLPVar

def getSolution(s, varGroupname):
    sol = s.getPrimalVariableSolution()
    return np.array([sol[i] for i in IndexFactory.varIndex[varGroupname]])


#def I(n):
#    '''
#    Return a sparse identity matrix of size *n*
#    '''
#    if n <= 0:
#        return None
#    return csc_matrixPlus(sparse.eye(n, n))

def checkComp(x, y):
    for i in range(len(x)):
        if x[i] != 0 and y[i] != 0:
            return i
    return -1

## A generic QP class
class QP:

    def __init__(self):
        # div by 2 just to avoid numeric errors
        self.infinity = sys.float_info.max / 2

    def ComputeObjectiveValue(self, x):
        return (0.5 * x * (x * self.G).T + np.dot(self.c, x) -
                self.objectiveOffset)

    def init(self, G, c, A, b, C, c_low, c_up, x_low, x_up):
        self.c = c.copy()
        self.A = A.copy()
        self.b = b.copy()
        self.n = G.shape[0]
        self.m = b.shape[0] + C.shape[0]
        self.nInEquality = C.shape[0]
        self.nEquality = b.shape[0]
        self.G = G.copy()
        self.C = C.copy()
        self.c_low = c_low.copy()
        self.c_up = c_up.copy()
        self.x_low = x_low.copy()
        self.x_up = x_up.copy()

    def sAll(self, x):
        '''
        Return Cx - b
        '''
        return self.A * x - self.b

    def s(self, x, i):
        '''
        Return C_ix - b_i
        :arg x: a vector
        :type x: Numpy array
        :arg i: index
        :type i: integer
        '''
        if i >= self.m:
            raise "i should be smaller than m"
        return self.A[i, :] * x - self.b[i]

    def getUnconstrainedSol(self):
        '''
        Return the unconstrained minimizer of the QP, :math:`-G^{-1}a`
        '''
        #G = self.L * self.L.T
        #Change the following line and use self.L
        return np.linalg.solve(G, -self.c)

    def gradient(self, x):
        '''
        Return the gradient of the objective function at ``x``
        :arg x: vector
        :type x: Numpy array
        '''
        return self.G * x + self.a

    def fromQps(self, filename):
        (self.G, self.c, self.A, self.b,
            self.C, self.c_low, self.c_up,
            self.x_low, self.x_up,
            self.n, self.nEquality, self.nInEquality,
            self.objectiveOffset) = readQPS(filename)
        self.filename = filename
        # Use this later if slack variable are added
        self.nOriginalVar = self.n

    def convertToEqualityOnly(self, varsToo=True):
        '''
        Add necessary slack and surplus variables and return the
        Augmented ``A`` and ``b``.
        '''
        A = self.A
        G = self.G
        b = CyLPArray(self.b)
        c = self.c
        C = self.C
        c_low = CyLPArray(self.c_low)
        c_up = CyLPArray(self.c_up)
        x_low = self.x_low
        x_up = self.x_up
        infinity = self.infinity

        nVar = self.n
        nEquality = self.nEquality
        nInEquality = self.nInEquality

#        if nEquality:
#            print('A')
#            print(A.todense())
#            print(b)
#
#        if nInEquality:
#            print(C')
#            print(C.todense())
#            print(c_low)
#            print(c_up)
#
#        print('Hessian')
#        print(G.todense())
#
#        print('c')
#        print(c)

        iVarsWithJustUpperBound = [i for i in range(nVar)
                            if x_up[i] < infinity and x_low[i] <= -infinity]
        nVarsWithJustUpperBound = len(iVarsWithJustUpperBound)

        iVarsWithJustLowerBound = [i for i in range(nVar)
                            if x_low[i] > -infinity and x_up[i] >= infinity]
        nVarsWithJustLowerBound = len(iVarsWithJustLowerBound)

        iVarsWithBothBounds = \
                [i for i in range(nVar)
                        if x_low[i] > -infinity and x_up[i] < infinity]
        nVarsWithBothBounds = len(iVarsWithBothBounds)

        iFreeVars = \
                [i for i in range(nVar)
                        if x_low[i] <= -infinity and x_up[i] >= infinity]
        nFreeVars = len(iFreeVars)

        iConstraintsWithBothBounds = \
                [i for i in range(nInEquality)
                        if c_up[i] < infinity and c_low[i] > -infinity]
        nConstraintsWithBothBounds = len(iConstraintsWithBothBounds)

        iConstraintsWithJustUpperBound = \
                [i for i in range(nInEquality)
                        if c_up[i] < infinity and c_low[i] <= -infinity]
        nConstraintsWithJustUpperBound = len(iConstraintsWithJustUpperBound)

        iConstraintsWithJustLowerBound = \
                [i for i in range(nInEquality)
                        if c_up[i] >= infinity and c_low[i] > -infinity]
        nConstraintsWithJustLowerBound = len(iConstraintsWithJustLowerBound)

#        print('<=C<=', len(iConstraintsWithBothBounds))
#        print('C<=', len(iConstraintsWithJustUpperBound))
#        print('<=C', len(iConstraintsWithJustLowerBound))
#        print('<=x<=', len(iVarsWithBothBounds))
#        print('x<=', len(iVarsWithJustUpperBound))
#        print('<=x', len(iVarsWithJustLowerBound))
#        print('free x', len(iFreeVars))
        #st = 'n, m, =, <c<, c<, <c, <x<, x<, <x, freeVar\n'


#        st = ''
#        ind = self.filename.rindex('/') + 1
#        st += self.filename[ind:] + ', '
#        st +=  '%g, ' % nVar
#        st +=  '%g, ' % (nEquality + nInEquality)
#        st +=  '%g, '% nEquality
#        st +=  '%g, '% len(iConstraintsWithBothBounds)
#        st +=  '%g, '% len(iConstraintsWithJustUpperBound)
#        st += '%g, '% len(iConstraintsWithJustLowerBound)
#        st += '%g, '% len(iVarsWithBothBounds)
#        st += '%g, '% len(iVarsWithJustUpperBound)
#        st += '%g, '% len(iVarsWithJustLowerBound)
#        st += '%g, '% len(iFreeVars)
#
#        st += '%g, ' % (G.nnz / float(nVar*nVar))
#        st += '%g, ' % (A.nnz / float(A.shape[0] * A.shape[1]))

        if nInEquality:
            iden = I(nInEquality)
            for i in iConstraintsWithJustLowerBound:
                iden[i, i] = -1
            C = sparseConcat(C, iden, 'h')

            c_rhs = np.zeros(nInEquality)

            if nConstraintsWithBothBounds:
                c_rhs[iConstraintsWithBothBounds] = \
                                    c_up[iConstraintsWithBothBounds]
                c_rhs = np.concatenate((c_rhs, c_up[iConstraintsWithBothBounds] -
                                    c_low[iConstraintsWithBothBounds]), axis=0)

                C = sparseConcat(C, iden[iConstraintsWithBothBounds, :], 'v', h_offset=-1)

                C = sparseConcat(C, I(nConstraintsWithBothBounds), 'h', v_offset=-1)

            if nConstraintsWithJustUpperBound:
                c_rhs[iConstraintsWithJustUpperBound] = \
                                    c_up[iConstraintsWithJustUpperBound]

            if nConstraintsWithJustLowerBound:
                c_rhs[iConstraintsWithJustLowerBound] = \
                                    c_low[iConstraintsWithJustLowerBound]
        else:
            C = None
            c_rhs = np.array([])

        if varsToo:
            V = None
            iNonFree = np.sort(np.concatenate((iVarsWithBothBounds,
                                       iVarsWithJustUpperBound,
                                       iVarsWithJustLowerBound), axis=0))
            nNonFree = len(iNonFree)
            iden = I(nVar)

            for i in iVarsWithJustLowerBound:
                iden[i, i] = -1

            if nNonFree:
                V = sparseConcat(V, I(nVar)[iNonFree, :], 'h')
                V = sparseConcat(V, iden[iNonFree, :][:, iNonFree],
                             'h', h_offset=nInEquality +
                                                nConstraintsWithBothBounds)


            v_rhs = np.zeros(nVar)

            if nVarsWithBothBounds:
                v_rhs[iVarsWithBothBounds] = \
                                    x_up[iVarsWithBothBounds]
                v_rhs = np.concatenate((v_rhs, x_up[iVarsWithBothBounds] -
                                    x_low[iVarsWithBothBounds]), axis=0)

                V = sparseConcat(V, iden[iVarsWithBothBounds, :][:, iNonFree],
                             'v', h_offset=-1)

                V = sparseConcat(V, I(nVarsWithBothBounds), 'h', v_offset=-1)

            if nVarsWithJustUpperBound:
                v_rhs[iVarsWithJustUpperBound] = \
                                    x_up[iVarsWithJustUpperBound]

            if nVarsWithJustLowerBound:
                v_rhs[iVarsWithJustLowerBound] = \
                                    x_low[iVarsWithJustLowerBound]

            if nFreeVars:
                v_rhs = np.delete(v_rhs, iFreeVars, 0)

            c_rhs = np.concatenate((c_rhs, v_rhs), axis=0)
            C = sparseConcat(C, V, 'v')


        if nEquality:
            C = sparseConcat(A, C, 'v')
            c_rhs = np.concatenate((b, c_rhs), axis=0)


#        mmm = C.todense()
#        from math import ceil
#        secSize = 25
#        cols = mmm.shape[1]
#        divs = int(ceil(cols / float(secSize)))
#
#        for sec in range(divs):
#            for i in range(mmm.shape[0]):
#                for j in range(sec * secSize, min((sec+1) * secSize, mmm.shape[1])):
#                    print(str(int(round(mmm[i, j], 1))).rjust(4),)
#                print()
#            print('...')
#
#        print(c_rhs)

        assert(C.shape[0] == len(c_rhs))
        self.A = C
        self.b = c_rhs



        self.C = None
        self.c_low = None
        self.c_up = None
        self.n = self.A.shape[1]
        # Resize the Hessian to match the new self.n
        self.G[self.n - 1, self.n - 1] = 0
        self.nEquality = self.A.shape[0]
        self.nInEquality = 0

        self.x_low = -infinity * np.ones(self.n)
        self.x_low[nVar:] = 0
        self.x_up = infinity * np.ones(self.n)

#        st += '%g, ' % (self.G.nnz / float(self.G.shape[0]*self.G.shape[0]))
#        st += '%g ' % (self.A.nnz /
#                            float(self.A.shape[0] * self.A.shape[1]))
#        st += '\n'
#        with open('qpstat', 'a') as f:
#            f.write(st)


    def WolfeEquality(self, method='w'):
        assert(self.nInEquality == 0)
        start = perf_counter()
        A = self.A
        b = CyLPArray(self.b)
        c = CyLPArray(self.c)
        G = self.G

#        print('A\n', A.todense())
#        print('b\n', b)
#        print('c\n', c)
#        print('G\n', G.todense())
#


        nVar = self.n
        nx = self.nOriginalVar
        nSlacks = nVar - nx


        m = CyLPModel()
        x = m.addVariable('x', nVar)
        m += A * x == b

        s = CyClpSimplex(m)

        if s.primal() != 'optimal':
            return



        sp = s.addVariable('sp', nVar)
        sm = s.addVariable('sm', nVar)
        z = s.addVariable('z', nSlacks)
        y = s.addVariable('y', self.nEquality)

        #from numpy import linalg as LA
        #print('cond:', LA.cond(G[:nx, :nx].todense()))
        #print('G nnz:', G.nnz, G.nnz / float(nx*nx))
        #print('A nnz:', A.nnz, A.nnz / float(A.shape[0] * A.shape[1]))

        #print(G.todense())
        minDiag = min(G[i, i] for i in range(nx))
        delta =  max(10**-8, 0.01 * max(1.0**-4, minDiag))
        G = G + delta * I(nVar)
        #print(G.todense())

#        print('G')
#        print(G.todense())
#
#        print('A')
#        print(A.todense())
#
#        print('A.T')
#        print(A.T.todense())
#
#        print('A.T[:nx, :]')
#        print(A.T[:nx, :].todense())
#
#        print('A.T[nx:, :]')
#        print(A.T[nx:, :].todense())
#
#        print(nx)

        s += G[:nx, :] * x - A.T[:nx, :] * y + sp[:nx] - sm[:nx] == -c[:nx]
        if nSlacks:
            s += -A.T[nx:, :] * y - z + sp[nx:] - sm[nx:] == 0
            s += z >= 0

        #s += A * x == b

        s += x[nx:] >= 0
        s += sp >= 0
        s += sm >= 0

        s.objective = sp.sum() + sm.sum()



        #s.writeMps('/Users/mehdi/Desktop/test.mps')

        p = WolfePivot(s)

        if method == 'wp':
            p = WolfePivotPE(s)

        if nSlacks:
            p.setComplement(m, x[nx:], z)
        #print('comp list:\n', p.complementarityList)

        s.setPivotMethod(p)
        timeToMake = perf_counter() - start
        start = perf_counter()
        s.primal()
        timeToSolve = perf_counter() - start

        self.writeReport('qpout', s, timeToMake, timeToSolve, method, p)

        print('x')
        print(s.primalVariableSolution['x'])
#        print('z')
#        print(s.primalVariableSolution['z'])
#        print('y')
#        print(s.primalVariableSolution['y'])
#
#        nx = self.nOriginalVar
#        x = np.matrix(s.primalVariableSolution['x']).T
#        y = np.matrix(s.primalVariableSolution['y']).T
#        z = np.matrix(s.primalVariableSolution['z']).T
#        #x = x[:nx]
#        #G = self.G[:nx, :nx]
#        c = np.matrix(self.c).T
#        print(c)
#        print(np.zeros((3, 1))))
#        c = np.concatenate((c, np.zeros((5, 1))), axis=0)
#        print(c)
#
#        print('A*x = ')
#        print(A * x)
#        print('G * x + c')
#        print(G * x + c)
#        print('A.T * y')
#        print(A.T * y)
#
#        print(G[:nx, :] * x - A.T[:nx, :] * y)
#        print(-A.T[nx:, :] * y - z)
#        #print(G * x + c - A.T * y - z)
        return

#        m = CyLPModel()
#        x = m.addVariable('x', nVar)
#        sp = m.addVariable('sp', nVar)
#        sm = m.addVariable('sm', nVar)
#        z = m.addVariable('z', nSlacks)
#        y = m.addVariable('y', self.nEquality)
#
#        m += G[:nx, :] * x - A.T[:nx, :] * y + sp[:nx] - sm[:nx] == -c[:nx]
#        m += -A.T[nx:, :] * y - z + sp[nx:] - sm[nx:] == 0
#
#        m += A * x == b
#
#        m += x[nx:] >= 0
#        m += z >= 0
#        m += sp >= 0
#        m += sm >= 0
#        m += z >= 0
#
#        m.objective = sp.sum() + sm.sum()
#
#        s = CyClpSimplex(m)
#
#
#        #s.writeMps('/Users/mehdi/Desktop/test.mps')
#
#        p = WolfePivot(s)
#
#        if method == 'wp':
#            p = WolfePivotPE(s)
#
#        p.setComplement(m, x[nx:], z)
#        #print('comp list:\n', p.complementarityList)
#
#        s.setPivotMethod(p)
##        timeToMake = perf_counter() - start
##        start = perf_counter()
#        s.primal()



#        timeToSolve = perf_counter() - start
        #s.initialPrimalSolve()
        if method == 'wp':
            total = p.compCount + p.nonCompCount
            print('comp : %g ' % (p.compCount / float(total)))
            print('comp rejection : %g' % (p.compRej / float(p.compCount)))
        #print(s.primalVariableSolution)
        print('OBJ:', s.objectiveValue)
        x = np.matrix(s.primalVariableSolution['x']).T
        #print(A * x - np.matrix(b).T)
        print('objective:')
        x = x[:nx]
        G = G[:nx, :nx]
        qobj = 0.5 * x.T * G * x + np.dot(c, x) - self.objectiveOffset
        print(qobj)
        print(s.primalVariableSolution)

#        print(s.iteration)
#        f = open('qpout', 'a')
#        st = '%s %s %s %s %s %s %s\n' % (self.filename.ljust(30), method.ljust(2),
#                str(round(s.objectiveValue, 5)).ljust(8),
#                str(round(qobj, 5)).ljust(8),
#                str(timeToMake),
#                str(timeToSolve),
#                str(timeToMake + timeToSolve))
#        f.write(st)
#        f.close()

    def writeReport(self, filename, s, timeToMake, timeToSolve, method, p):

        def getcell(s, r, j):
            return str(round(s, r)).ljust(j)

        nx = self.nOriginalVar
        x = np.matrix(s.primalVariableSolution['x']).T
        x = x[:nx]
        G = self.G[:nx, :nx]
        c = self.c

        qobj = 0.5 * x.T * G * x + np.dot(c, x) - self.objectiveOffset

        if method == 'wp':
            total = p.compCount + p.nonCompCount
            compPer = (p.compCount / float(total))
            compRejPer = (p.compRej / float(p.compCount))

            st = '%s %s %s %s %s %s %d %s %s\n' % (self.filename.ljust(30),
                method.ljust(2),
                getcell(s.objectiveValue, 5, 8),
                getcell(qobj, 5, 8),
                getcell(timeToMake, 3, 8),
                getcell(timeToSolve, 3, 8),
                s.iteration,
                getcell(compPer, 2, 5),
                getcell(compRejPer, 2, 5))
        else:
            st = '%s %s %s %s %s %s %d\n' % (self.filename.ljust(30), method.ljust(2),
                getcell(s.objectiveValue, 5, 8),
                getcell(qobj, 5, 8),
                getcell(timeToMake, 3, 8),
                getcell(timeToSolve, 3, 8),
                s.iteration)

        with open('qpout', 'a') as f:
            f.write(st)

    def Wolfe_2(self):
        '''
        Solve a QP using Wolfe's method
        '''

        A = self.A
        G = self.G
        b = self.b
        c = self.c
        C = self.C
        c_low = self.c_low
        c_up = self. c_up
        x_low = self.x_low
        x_up = self.x_up
        infinity = self.infinity

        nVar = self.n
        nEquality = self.nEquality
        nInEquality = self.nInEquality
        nCons = nEquality + nInEquality

        varIndexDic = {}
        currentNumberOfVars = 0
        constIndexDic = {}
        currentNumberOfConst = 0

        s = CyClpSimplex()
        model = CyCoinModel()

        inds = range(nVar)

        # Convert G and A to sparse matrices (coo) if they're not already
#       if type(G) == np.matrixlib.defmatrix.matrix:
#           temp = sparse.lil_matrix(G.shape)
#           temp[:, :] = G
#           G = temp.tocoo()
#
#       if nEquality != 0 and type(A) == np.matrixlib.defmatrix.matrix:
#           temp = sparse.lil_matrix(A.shape)
#           temp[:, :] = A
#           A = temp.tocoo()
#
#       if nInEquality != 0 and type(C) == np.matrixlib.defmatrix.matrix:
#           temp = sparse.lil_matrix(C.shape)
#           temp[:, :] = C
#           C = temp.tocoo()

        #i for indices, n for size of the set
        iVarsWithUpperBound = \
                [i for i in range(len(x_up)) if x_up[i] < infinity]
        nVarsWithUpperBound = len(iVarsWithUpperBound)

        iVarsWithLowerBound = \
                [i for i in range(len(x_low)) if x_low[i] > -infinity]
        nVarsWithLowerBound = len(iVarsWithLowerBound)

        iVarsWithBothBounds = \
                [i for i in range(len(x_low)) if
                 x_low[i] > -infinity and x_low[i] < infinity]
        nVarsWithBothBounds = len(iVarsWithBothBounds)

        iConstraintsWithBothBounds = \
                [i for i in range(nInEquality)
                        if c_up[i] < infinity and c_low[i] > -infinity]
        nConstraintsWithBothBounds = len(iConstraintsWithBothBounds)

        iConstraintsWithJustUpperBound = \
                [i for i in range(nInEquality)
                        if c_up[i] < infinity and c_low[i] <= -infinity]
        nConstraintsWithJustUpperBound = len(iConstraintsWithJustUpperBound)

        iConstraintsWithJustLowerBound = \
                [i for i in range(nInEquality)
                        if c_up[i] >= infinity and c_low[i] > -infinity]
        nConstraintsWithJustLowerBound = len(iConstraintsWithJustLowerBound)

        iVarsWithUpperBound = [i for i in inds if x_up[i] < infinity]
        nVarsWithUpperBound = len(iVarsWithUpperBound)

        iVarsWithLowerBound = [i for i in inds if x_low[i] > -infinity]
        nVarsWithLowerBound = len(iVarsWithLowerBound)

        return


        # AA for Augmented A
        # number of elements of x, g1, g2, g3, x_u, x_l, g_u, s^+, s^-
        nColsAA = nVar + \
                    nInEquality + \
                    nVarsWithLowerBound + \
                    nVarsWithUpperBound + \
                    nConstraintsWithUpperLowerBound + \
                    nVar + \
                    nVar

        s.resize(0, nColsAA)

        #adding Ax = b
        zeroArray = np.zeros(nVar)
        infinityArray = infinity + zeroArray
        #minusInfinityArray = -infinity + zeroArray

        if nEquality:
            #s.addColumns(A.getnnz(), zeroArray,
            #        infinityArray, zeroArray, A.indptr, A.indices, A.data)
            s.addRows(nEquality, b, b, A.indptr, A.indices, A.data)

        if nInEquality:
            CRhs = np.zeros(nInEquality)
            nCol_C = C.shape[1]
            for i in range(nInEquality):
                if i in iConstraintsWithUpperLowerBound:
                    C[i, nCol_C + i] = 1
                    CRhs[i] = 1
                elif i in iConstraintsWithJustUpperBound:
                    C[i, nCol_C + i] = 1
                    CRhs[i] = 1
                else:
                    C[i, nCol_C + i] = -1
                    CRhs[i] = -1

            s.addRows(nInEquality, CRhs, CRhs, C.indptr, C.indices, C.data)

        return

        #constructing A for Ax = b
        # step one: concatenating C to the bottom

        #but first adding the indices of Ax=b to the Index Factory
        IndexFactory.addConst('Ax=b', nEquality)
        IndexFactory.addVar('x', nVar)

        for i in range(nInEquality):
            rowi = C.getrow(i).tocoo()
            if c_up[i] < infinity:
                rowi = sparseConcat(rowi, e_sparse(i, nInEquality, 1), 'h')
                A = sparseConcat(A, rowi, 'v')
                #if i in c_low.keys():
                if c_low[i] > -infinity:
                    b = np.concatenate((b, np.array([c_up[i]])), axis=1)
                    IndexFactory.addConst('C_1X+g1=c_up', 1)
                    IndexFactory.addVar('g1', 1)
                else:
                    b = np.concatenate((b, np.array([c_up[i]])), axis=1)
                    IndexFactory.addConst('C_2X+g2=c_up', 1)
                    IndexFactory.addVar('g2', 1)
            else:
                rowi = sparseConcat(rowi, e_sparse(i, nInEquality, -1), 'h')
                A = sparseConcat(A, rowi, 'v')
                b = np.concatenate((b, np.array([c_low[i]])), axis=1)
                IndexFactory.addConst('C_3X-g3=c_low', 1)
                IndexFactory.addVar('g3', 1)

            #whatever the variable (g1,g2 or g3) >=0

        ##step two: adding x + x_u <= U and x - x_l >= L
        x_up_count = 0

        lengthOfRowToAdd = IndexFactory.currentVarIndex + \
                                        len(iVarsWithUpperBounds)
        startInd = IndexFactory.currentVarIndex
        for i in iVarsWithUpperBounds:
                rowToAdd = e_sparse(i, lengthOfRowToAdd, 1)
                rowToAdd[0, startInd + x_up_count] = 1
                x_up_count += 1
                A = sparseConcat(A, rowToAdd, 'v')
                b = np.concatenate((b, np.array([x_up[i]])), axis=1)
                IndexFactory.addConst('x+x_u=x_up', 1)
                IndexFactory.addVar('x_u', 1)

        lengthOfRowToAdd = IndexFactory.currentVarIndex + \
                                            len(iVarsWithLowerBounds)

        startInd = IndexFactory.currentVarIndex
        x_low_count = 0
        #for i in x_low.keys():
        for i in iVarsWithLowerBounds:
                rowToAdd = e_sparse(i, lengthOfRowToAdd, 1)
                rowToAdd[0,  startInd + x_low_count] = -1
                x_low_count += 1
                A = sparseConcat(A, rowToAdd, 'v')
                b = np.concatenate((b, np.array([x_low[i]])), axis=1)
                IndexFactory.addConst('x-x_l=x_low', 1)
                IndexFactory.addVar('x_l', 1)

        if 'g1' in IndexFactory.varIndex.keys():
            g1_inds = IndexFactory.varIndex['g1']
            for i in range(len(g1_inds)):
                g1_i = g1_inds[i]
                rowToAdd = e_sparse(g1_i,
                                IndexFactory.currentVarIndex + len(g1_inds), 1)
                rowToAdd[0, IndexFactory.currentVarIndex + i] = 1
                A = sparseConcat(A, rowToAdd, 'v')
                indexInC = IndexFactory.constIndex['C_1X+g1=c_up'][i] - \
                                                            nEquality
                b = np.concatenate((b,
                        np.array([c_up[indexInC] - c_low[indexInC]])), axis=1)
                IndexFactory.addConst('g_1+g_u=c_up-c_low', 1)
                IndexFactory.addVar('g_u', 1)

        #(Augmented A) x = b
        for i in range(A.shape[0]):
            rowi = A.getrow(i).tocoo()
            model.addConstraint(rowi.nnz,
                    np.array(rowi.col, np.int32),
                    np.array(rowi.data, 'd'), b[i], b[i])

        setPositive(model, 'g1')
        setPositive(model, 'g2')
        setPositive(model, 'g3')
        setPositive(model, 'x_u')
        setPositive(model, 'x_l')
        setPositive(model, 'g_u')

        #Gx = -c
        for i in range(nVar):
            rowi = G.getrow(i).tocoo()
            model.addConstraint(rowi.nnz,
                    np.array(rowi.col, np.int32),
                    np.array(rowi.data, np.double), -c[i], -c[i])
        IndexFactory.addConst('Gx=-c', nVar)

        for i in range(nVar):
            model.setVariableLower(i, -infinity)

        #adding s^+    so we have: GX + s^+ = -c
        for i in range(nVar):
            model.addVariable(1,
                        np.array([IndexFactory.constIndex['Gx=-c'][i]],
                        np.int32), np.array([1.], np.double),
                        0, infinity, 0)
        IndexFactory.addVar('s^+', nVar)

        #adding s^-    now: GX + s^+ - s^- = -c
        for i in range(nVar):
            model.addVariable(1,
                        np.array([IndexFactory.constIndex['Gx=-c'][i]],
                        np.int32),
                        np.array([-1.], np.double), 0, infinity, 0)
        IndexFactory.addVar('s^-', nVar)

        # We can run the primal here to find a feasible
        # point to start the second phase
        # as in Wolfe. But in practice, doesn't seem like a good idea
        #s.loadProblem(model, 0)
        #s.primal()

        #adding extra 0's to G for variables other than x: g1, g2, x_u,...

        # adding rhs=0 for A^Ty constraints corresponding
        # to all variables except 'x'
        # These are the variables for which G.row(i) and c_i are zero
        # we don't know how many they are as, for example, we may or may not
        # have Cx<=c_up or Cx >= c_low or ...
        for i in range(A.shape[1] - nVar):
            model.addConstraint(0, np.array([], np.int32),
                                np.array([], np.double), 0, 0)

        #adding -A^Ty_A
        for i in range(A.shape[0]):
            rowi = -A.getrow(i).tocoo()
            #print('rowi : ')
            #print(rowi.col)
            cols = rowi.col + IndexFactory.constIndex['Gx=-c'][0]
            #print(cols)
            model.addVariable(rowi.nnz, np.array(cols, np.int32),
                            np.array(rowi.data, 'd'), -infinity, infinity, 0)
        IndexFactory.addVar('y_A', A.shape[0])

        cl = np.array(range(IndexFactory.currentVarIndex +
                            A.shape[1]), np.int32)
        s.setComplementarityList(cl)

        #adding -z
        startRow = IndexFactory.constIndex['Gx=-c'][0]
        #TODO: check this range
        #for i in range(nVar, A.shape[1]):
        for i in range(nVar, A.shape[1]):
            model.addVariable(1, np.array([startRow + i], np.int32),
                            np.array([-1.], 'd'), 0, infinity, 0)
            IndexFactory.addVar('z', 1)
            compind = IndexFactory.getLastVarIndex()
            s.setComplement(i, compind)
            #cl[i] , cl[compind] = compind, i

        #setting the objective coefficients of s^+ to one,
        sPlusIndex = IndexFactory.varIndex['s^+']
        sMinusIndex = IndexFactory.varIndex['s^-']
        for i in sPlusIndex:
            model.setObjective(i, 1)

        #setting the objective coefficients of s^- to one,
        for i in sMinusIndex:
            model.setObjective(i, 1)

        s.loadProblem(model, 0)

        # I think that preSolve could be harmful here.
        # what happens to the complement variables?
        # s = s.preSolve(feasibilityTolerance = 10**-8)
        # s.setComplementarityList(cl)

        # This means that we want to use IClpSimplexPrimal_Wolfe
        # instead of ClpSimplexPrimal
        s.useCustomPrimal(1)

        #this means that we want to use cythons's Cywolfe
        #s.setPrimalColumnPivotAlgorithmToWolfe()
        #p = WolfePivot(s, bucketSize=float(sys.argv[2]))

        p = PositiveEdgeWolfePivot(s, bucketSize=float(sys.argv[2]))
        s.setPivotMethod(p)

        st = perf_counter()
        s.primal()
        print("CLP time : %g seconds" % (perf_counter() - st))

        x = s.getPrimalVariableSolution()
        print("sol = ")
        x = x[:nVar]
        #print(x)
        G = G.todense()
        #print(G.shape)
        #print(x * G)
        #print(c)
        #print(x)
        print(0.5 * x * (x * G).T + np.dot(c, x) - self.objectiveOffset)

        return

    def Wolfe(self, method='w'):
        '''
        Solves a QP using Wolfe's method (``method = 'w'``) or Wolfe's method using
        positive edge as pivot rule (``method = 'wp'``).
        '''
        start = perf_counter()
        A = self.A
        G = self.G
        b = CyLPArray(self.b)
        c = self.c
        C = self.C
        c_low = CyLPArray(self.c_low)
        c_up = CyLPArray(self.c_up)
        x_low = self.x_low
        x_up = self.x_up
        infinity = self.infinity

        nVar = self.n
        nEquality = self.nEquality
        nInEquality = self.nInEquality

#        print('A')
#        print(A)
#        print(b)
#
#        print('C')
#        print(C)
#        print(c_low)
#        print(c_up)
#        print('Hessian')
#        print(G)
#
#        print('c')
#        print(c)

        iVarsWithJustUpperBound = [i for i in range(nVar)
                            if x_up[i] < infinity and x_low[i] <= -infinity]
        nVarsWithJustUpperBound = len(iVarsWithJustUpperBound)

        iVarsWithJustLowerBound = [i for i in range(nVar)
                            if x_low[i] > -infinity and x_up[i] >= infinity]
        nVarsWithJustLowerBound = len(iVarsWithJustLowerBound)

        iVarsWithBothBounds = \
                [i for i in range(nVar)
                        if x_low[i] > -infinity and x_up[i] < infinity]
        nVarsWithBothBounds = len(iVarsWithBothBounds)

        iConstraintsWithBothBounds = \
                [i for i in range(nInEquality)
                        if c_up[i] < infinity and c_low[i] > -infinity]
        nConstraintsWithBothBounds = len(iConstraintsWithBothBounds)

        iConstraintsWithJustUpperBound = \
                [i for i in range(nInEquality)
                        if c_up[i] < infinity and c_low[i] <= -infinity]
        nConstraintsWithJustUpperBound = len(iConstraintsWithJustUpperBound)

        iConstraintsWithJustLowerBound = \
                [i for i in range(nInEquality)
                        if c_up[i] >= infinity and c_low[i] > -infinity]
        nConstraintsWithJustLowerBound = len(iConstraintsWithJustLowerBound)

#        print('____________________________________________')
#        print(x_up)
#        print(x_low)
#
#        print(c_low)
#        print(c_up)
#
#        print(C.todense())
#        print(iConstraintsWithBothBounds)
#        print(iConstraintsWithJustLowerBound)
#        print(iConstraintsWithJustUpperBound)
#
#        print('vars')
#        print(iVarsWithBothBounds)
#        print(iVarsWithJustLowerBound)
#        print(iVarsWithJustUpperBound)
#        print('____________________________________________')


        In = I(nVar)

        m = CyLPModel()
        x = m.addVariable('x', nVar)

        yx1 = CyLPVar('yx1', dim=0)
        yxu = CyLPVar('yxu', dim=0)
        yx2 = CyLPVar('yx2', dim=0)
        yx3 = CyLPVar('yx3', dim=0)
        if nVarsWithBothBounds:
            k1 = m.addVariable('k1', nVarsWithBothBounds)
            zk1 = m.addVariable('zk1', nVarsWithBothBounds)
            spk1 = m.addVariable('spk1', nVarsWithBothBounds)
            smk1 = m.addVariable('smk1', nVarsWithBothBounds)
            yx1 = m.addVariable('yx1', nVarsWithBothBounds)
            m.addConstraint(k1 >= 0)
            m.addConstraint(spk1 >= 0)
            m.addConstraint(smk1 >= 0)
            m.addConstraint(zk1 >= 0)
            ku = m.addVariable('ku', nVarsWithBothBounds)
            zku = m.addVariable('zku', nVarsWithBothBounds)
            spku = m.addVariable('spku', nVarsWithBothBounds)
            smku = m.addVariable('smku', nVarsWithBothBounds)
            yxu = m.addVariable('yxu', nVarsWithBothBounds)
            m.addConstraint(ku >= 0)
            m.addConstraint(zku >= 0)
            m.addConstraint(spku >= 0)
            m.addConstraint(smku >= 0)
            m.addConstraint(x[iVarsWithBothBounds] + k1 ==
                            x_up[iVarsWithBothBounds], 'x1+k1')
            m.addConstraint(k1 + ku ==
                    (x_up[iVarsWithBothBounds] -
                                        x_low[iVarsWithBothBounds]), 'k1+ku')

        if nVarsWithJustUpperBound:
            k2 = m.addVariable('k2', nVarsWithJustUpperBound)
            zk2 = m.addVariable('zk2', nVarsWithJustUpperBound)
            #spk2 = m.addVariable('spk2', nVarsWithJustUpperBound)
            #smk2 = m.addVariable('smk2', nVarsWithJustUpperBound)
            yx2 = m.addVariable('yx2', nVarsWithJustUpperBound)
            m.addConstraint(k2 >= 0)
            m.addConstraint(zk2 >= 0)
            #m.addConstraint(spk2 >= 0)
            #m.addConstraint(smk2 >= 0)
            m.addConstraint(x[iVarsWithJustUpperBound] + k2 ==
                            x_up[iVarsWithJustUpperBound], 'x2+k2')

        if nVarsWithJustLowerBound:
            k3 = m.addVariable('k3', nVarsWithJustLowerBound)
            zk3 = m.addVariable('zk3', nVarsWithJustLowerBound)
            #spk3 = m.addVariable('spk3', nVarsWithJustLowerBound)
            #smk3 = m.addVariable('smk3', nVarsWithJustLowerBound)
            yx3 = m.addVariable('yx3', nVarsWithJustLowerBound)
            m.addConstraint(k3 >= 0)
            m.addConstraint(zk3 >= 0)
            #m.addConstraint(spk3 >= 0)
            #m.addConstraint(smk3 >= 0)
            m.addConstraint(x[iVarsWithJustLowerBound] -  k3 ==
                            x_low[iVarsWithJustLowerBound], 'x3-k3')
#            m.addConstraint(I(nVarsWithJustLowerBound) *
#                                    x[iVarsWithJustLowerBound] -
#                            I(nVarsWithJustLowerBound) * k3 ==
#                            x_low[iVarsWithJustLowerBound], 'x3-k3')

        if nEquality > 0 :
            m.addConstraint(A * x == b, 'Ax=b')
            yb = m.addVariable('yb', nEquality)

        C1T = C2T = C3T = None
        yc1 = CyLPVar('yc1', dim=0)
        ycu = CyLPVar('ycu', dim=0)
        yc2 = CyLPVar('yc2', dim=0)
        yc3 = CyLPVar('yc3', dim=0)
        if nConstraintsWithBothBounds:
            C1 = C[iConstraintsWithBothBounds, :]
            C1T = C1.T
            # g1: slack for ineq. const. with both bounds
            g1 = m.addVariable('g1', nConstraintsWithBothBounds)
            zg1 = m.addVariable('zg1', nConstraintsWithBothBounds)
            spg1 = m.addVariable('spg1', nConstraintsWithBothBounds)
            smg1 = m.addVariable('smg1', nConstraintsWithBothBounds)
            yc1 = m.addVariable('yc1', nConstraintsWithBothBounds)
            m.addConstraint(g1 >= 0)
            m.addConstraint(spg1 >= 0)
            m.addConstraint(smg1 >= 0)
            m.addConstraint(zg1 >= 0)
            # gu: slack of g1 from c_up - c_low
            gu = m.addVariable('gu', nConstraintsWithBothBounds)
            zgu = m.addVariable('zgu', nConstraintsWithBothBounds)
            spgu = m.addVariable('spgu', nConstraintsWithBothBounds)
            smgu = m.addVariable('smgu', nConstraintsWithBothBounds)
            ycu = m.addVariable('ycu', nConstraintsWithBothBounds)
            m.addConstraint(gu >= 0)
            m.addConstraint(zgu >= 0)
            m.addConstraint(spgu >= 0)
            m.addConstraint(smgu >= 0)
            m.addConstraint(C1 * x +
                            g1 == #+ spg1 - smg1 ==
                            c_up[iConstraintsWithBothBounds],
                            'C1x+g1')
            m.addConstraint(g1 + gu == #+ spgu - smgu ==
                            (c_up[iConstraintsWithBothBounds] -
                             c_low[iConstraintsWithBothBounds]),
                             'g1+gu')


        if nConstraintsWithJustUpperBound:
            C2 = C[iConstraintsWithJustUpperBound, :]
            C2T = C2.T
            g2 = m.addVariable('g2', nConstraintsWithJustUpperBound)
            zg2 = m.addVariable('zg2', nConstraintsWithJustUpperBound)
            #spg2 = m.addVariable('spg2', nConstraintsWithJustLowerBound)
            #smg2 = m.addVariable('smg2', nConstraintsWithJustLowerBound)
            yc2 = m.addVariable('yc2', nConstraintsWithJustUpperBound)
            m.addConstraint(g2 >= 0)
            m.addConstraint(zg2 >= 0)
            #m.addConstraint(spg2 >= 0)
            #m.addConstraint(smg2 >= 0)
            m.addConstraint(C2 * x + g2 == #+ spg2 - smg2 ==
                        c_up[iConstraintsWithJustUpperBound],
                                'C2x+g2')
        if nConstraintsWithJustLowerBound:
            C3 = C[iConstraintsWithJustLowerBound, :]
            C3T = C3.T
            g3 = m.addVariable('g3', nConstraintsWithJustLowerBound)
            zg3 = m.addVariable('zg3', nConstraintsWithJustLowerBound)
            spg3 = m.addVariable('spg3', nConstraintsWithJustLowerBound)
            smg3 = m.addVariable('smg3', nConstraintsWithJustLowerBound)
            yc3 = m.addVariable('yc3', nConstraintsWithJustLowerBound)
            m.addConstraint(g3 >= 0)
            m.addConstraint(zg3 >= 0)
            m.addConstraint(spg3 >= 0)
            m.addConstraint(smg3 >= 0)
            m.addConstraint(C3 * x -
                                g3 == #+ spg3 - smg3  ==
                                c_low[iConstraintsWithJustLowerBound],
                                'C3x-g3')


        x1CoefT = x2CoefT = x3CoefT = xuCoefT = None
        if nVarsWithBothBounds:
            x1CoefT = In[iVarsWithBothBounds, :].T
            xuCoefT = In[iVarsWithBothBounds, :].T
            Ik = I(nVarsWithBothBounds)
            k1CoefT = Ik
            kuCoefT = Ik


        if nVarsWithJustUpperBound:
            x2CoefT =  In[iVarsWithJustUpperBound, :].T
            Ik = I(nVarsWithJustUpperBound)
            k2CoefT =  Ik

        if nVarsWithJustLowerBound:
            x3CoefT = In[iVarsWithJustLowerBound, :].T
            Ik = I(nVarsWithJustLowerBound)
            k3CoefT = Ik

        sp = m.addVariable('sp', nVar)
        m.addConstraint(sp >= 0)
        sm = m.addVariable('sm', nVar)
        m.addConstraint(sm >= 0)

        #z = m.addVariable('z', nVar)


        # Dual-feasibility constraints:
        if nEquality:
            #from pudb import set_trace; set_trace()
            m.addConstraint(G * x - A.T * yb - C1T * yc1 - C2T * yc2 - C3T * yc3 -
                        x1CoefT * yx1 - x2CoefT * yx2 - x3CoefT * yx3 + sp - sm  == -c,
                        'Gx-ATy-CTu-z')
        else:
            m.addConstraint(G * x - C1T * yc1 - C2T * yc2 - C3T * yc3 -
                        x1CoefT * yx1 - x2CoefT * yx2 - x3CoefT * yx3 + sp - sm  == -c ,
                        'Gx-CTu-z')


        if nInEquality:
            #Im = I(nInEquality)

            if nConstraintsWithBothBounds:
                g1Coef = I(nConstraintsWithBothBounds)
                guCoef = I(nConstraintsWithBothBounds)
                #g1Coef = Im[:nConstraintsWithBothBounds, :]
                #guCoef = Im[:nConstraintsWithBothBounds, :]
                #g1Coef = Im[iConstraintsWithBothBounds, :]
                #guCoef = Im[iConstraintsWithBothBounds, :]
                #g1Coef = I(nConstraintsWithBothBounds)
                #guCoef = I(nConstraintsWithBothBounds)
                m.addConstraint(-g1Coef.T * yc1 - g1Coef.T * ycu - zg1 +
                                spg1 - smg1 == 0,
                                'dualfeas_g1')
                m.addConstraint(-guCoef.T * ycu - zgu +
                                 spgu - smgu == 0,
                                'dualfeas_gu')

            if nConstraintsWithJustUpperBound:
                g2Coef = I(nConstraintsWithJustUpperBound)
                #g2Coef = Im[nConstraintsWithBothBounds :
                #             (nConstraintsWithBothBounds +
                #                 nConstraintsWithJustUpperBound), :]
                #g2Coef = Im[iConstraintsWithJustUpperBound, :]
                #g2Coef = I(nConstraintsWithJustUpperBound)
                m.addConstraint(-g2Coef.T * yc2 - zg2 == 0, 'dualfeas_g2')

            if nConstraintsWithJustLowerBound:
                g3Coef = I(nConstraintsWithJustLowerBound)
                #g3Coef = Im[(nConstraintsWithBothBounds +
                #             nConstraintsWithJustUpperBound):, :]
                #g3Coef = Im[iConstraintsWithJustLowerBound, :]
                #g3Coef = I(nConstraintsWithJustLowerBound)
                m.addConstraint(g3Coef.T * yc3 - zg3 + spg3 - smg3 == 0, 'dualfeas_g3')


        if nVarsWithBothBounds:
            m.addConstraint(-k1CoefT * yx1 - k1CoefT * yxu - zk1 + \
                            spk1 - smk1 == 0,
                            'dualfeas_k1')
            m.addConstraint(-kuCoefT * yxu - zku + spku - smku == 0, 'dualfeas_ku')

        if nVarsWithJustUpperBound:
            m.addConstraint(-k2CoefT * yx2 - zk2 == 0, 'dualfeas_k2')

        if nVarsWithJustLowerBound:
            #m.addConstraint(x3CoefT * yx3 - zk3 + spk3 - smk3 == 0, 'dualfeas_k3')
            m.addConstraint(k3CoefT * yx3 - zk3 == 0, 'dualfeas_k3')


#        m.objective =  sp.sum() + sm.sum() + spg1.sum() + smg1.sum() + \
#                        spgu.sum() + smgu.sum() + spg3.sum() + smg3.sum() + \
#                        spk1.sum() + smk1.sum() + spku.sum() + smku.sum()
        m.objective =  sp.sum() + sm.sum() + spg1.sum() + smg1.sum() + \
                        spgu.sum() + smgu.sum()
        #z = m.addVariable('z', nVar)

        s = CyClpSimplex(m)
        ###s.setComplement(x, z)
        #print(m.inds)
        #s.writeMps('/Users/mehdi/Desktop/test.mps')
        #s.useCustomPrimal(True)


        p = WolfePivot(s)

        if method == 'wp':
            p = WolfePivotPE(s)

        if nConstraintsWithBothBounds:
            p.setComplement(m, g1, zg1)
            p.setComplement(m, gu, zgu)

        if nConstraintsWithJustUpperBound:
            p.setComplement(m, g2, zg2)

        if nConstraintsWithJustLowerBound:
            p.setComplement(m, g3, zg3)
            #p.setComplement(m, sc3, yc3)


        if nVarsWithBothBounds:
            p.setComplement(m, k1, zk1)
            p.setComplement(m, ku, zku)

        if nVarsWithJustUpperBound:
            p.setComplement(m, k2, zk2)

        if nVarsWithJustLowerBound:
            p.setComplement(m, k3, zk3)

        #print(p.complementarityList)
        s.setPivotMethod(p)
        timeToMake = perf_counter() - start
        start = perf_counter()
        s.primal()
        timeToSolve = perf_counter() - start
        #s.initialPrimalSolve()
        if method == 'wp':
            total = p.compCount + p.nonCompCount
            print('comp : %g ' % (p.compCount / float(total)))
            print('comp rejection : %g' % (p.compRej / float(total)))
        #print(s.primalVariableSolution)
        print('OBJ:', s.objectiveValue)
        x = np.matrix(s.primalVariableSolution['x']).T
        print('objective:')
        qobj = 0.5 * x.T * G * x + np.dot(c, x) - self.objectiveOffset

        print(s.iteration)
        with open('qpout', 'a') as f:
            st = '%s %s %s %s %s %s %s\n' % (self.filename.ljust(30), method.ljust(2),
                str(round(s.objectiveValue, 5)).ljust(8),
                str(round(qobj, 5)).ljust(8),
                str(timeToMake),
                str(timeToSolve),
                str(timeToMake + timeToSolve))
            f.write(st)

#        print(checkComp(s.primalVariableSolution['k1'],
#                        s.primalVariableSolution['zk1']))
#
#        print(checkComp(s.primalVariableSolution['ku'],
#                        s.primalVariableSolution['zku']))

#        print('A:')
#        print(A)
#        print('x')
#        print(x)
#        print('Ax:')
#        print(A * x)
#        print('b')
#        print(b)
#        y = np.matrix(s.primalVariableSolution['yb']).T
#        #yx1 = s.primalVariableSolution['yx1']
#
#        print(A.shape)
#        print(((A * x).T - b < 10 ** -5).all())
#
#        print('opt')
#        print(G.todense())
#        print(np.linalg.eigvals(G.todense()))
#        print(G * x - A.T * y + np.matrix(c).T)
#        x = s.primalVariableSolution['x']
#
#        for i in range(nVar):
#            if x[i] > x_up[i] + 10**-10 or x[i] < x_low[i] - 10**-10:
#                print(i, 'INFEASIBLE', x_low[i] , x[i], x_up[i], x[i] > x_up[i], x[i] < x_low[i])

        #print('Cx')
        #print(C * x)
        #print('g3')
        #print(s.primalVariableSolution['g3'])
        #print('Cx - g3')
        #print(C * x - np.matrix(s.primalVariableSolution['g3']).T)
        #print('optimality:')
        #print(-c)
        #y =  s.primalVariableSolution['yc3']
        #print(G*x)
        #print(C.T * y)
        #print('opt:')
        #print((G * x).T - C.T * y + c)

#        print('-------------------------------------------------------')
#        x = np.matrix([[1, 2, -1, 3, -4]]).T
#        print(0.5 * x.T * G * x + np.dot(c, x) - self.objectiveOffset
#        print('Cx')
#        print(C * x)
#        print('Cx - c_low')
#        print(C * x - np.matrix(c_low).T)


        return
        m.addConstraint(z >= 0)

        C = C.todense()
        G = G.todense()
        if nEquality > 0 :
            m.addConstraint(A * x == b)
            y = m.addVariable('y', nEquality)

        if nInEquality > 0 :
            c_low = CyLPArray(c_low)
            c_up = CyLPArray(c_up)
            print(C)
            #c_low *= -1
            #m.addConstraint(c_low <= C * x <= c_up)
            #ss = m.addVariable('ss', 1)
            m.addConstraint(-C * x  == -c_low)
            #m.addConstraint(-C * x <= -c_low)

            u = m.addVariable('u', nInEquality)
            #sp = m.addVariable('sp', nVar)
            #sm = m.addVariable('sm', nVar)
            #s = m.addVariable('s', 1)
            #m.addConstraint(ss >= 0)
            #m.addConstraint(sp >= 0)
            #m.addConstraint(sm >= 0)
            m.addConstraint(u >= 0)
            #m.addConstraint(s >= 0)
            #m.addConstraint(C * x  - s == c_low)


        if nEquality > 0:
            if nInEquality == 0:
                m.addConstraint(G * x - A.T * y - z == -c)
            else:
                m.addConstraint(G * x - A.T * y - C.T * u - z == -c)
        elif nInEquality > 0:
#            print(G.shape)
#            print(x.dim)
#            print(C.T)
#            print(u.dim)
#            print(c.shape)
#            print(z.dim)
            m.addConstraint(G * x - z - C.T * u + sp - sm == -c)  #- C.T * u - z

        m.objective = sm + sp

        s = CyClpSimplex(m)
        return
        #s.setComplement(x, z)
        s.useCustomPrimal(True)

        p = WolfePivot(s)
        s.setPivotMethod(p)

        s.primal()
        #s.initialPrimalSolve()
        print(s.primalVariableSolution)
        print(s.objectiveValue)
        return

        varIndexDic = {}
        currentNumberOfVars = 0
        constIndexDic = {}
        currentNumberOfConst = 0

        s = CyClpSimplex()
        model = CyCoinModel()

        inds = range(nVar)

        # Convert G and A to sparse matrices (coo) if they're not already
        if type(G) == np.matrixlib.defmatrix.matrix:
            temp = sparse.lil_matrix(G.shape)
            temp[:, :] = G
            G = temp.tocoo()

        if nEquality != 0 and type(A) == np.matrixlib.defmatrix.matrix:
            temp = sparse.lil_matrix(A.shape)
            temp[:, :] = A
            A = temp.tocoo()

        if nInEquality != 0 and type(C) == np.matrixlib.defmatrix.matrix:
            temp = sparse.lil_matrix(C.shape)
            temp[:, :] = C
            C = temp.tocoo()

        #constructing A for Ax = b
        # step one: concatenating C to the bottom

        #but first adding the indices of Ax=b to the Index Factory
        IndexFactory.addConst('Ax=b', nEquality)
        IndexFactory.addVar('x', nVar)

        for i in range(nInEquality):
            rowi = C.getrow(i).tocoo()
            if c_up[i] < infinity:
                rowi = sparseConcat(rowi, e_sparse(i, nInEquality, 1), 'h')
                A = sparseConcat(A, rowi, 'v')
                #if i in c_low.keys():
                if c_low[i] > -infinity:
                    b = np.concatenate((b, np.array([c_up[i]])), axis=1)
                    IndexFactory.addConst('C_1X+g1=c_up', 1)
                    IndexFactory.addVar('g1', 1)
                else:
                    b = np.concatenate((b, np.array([c_up[i]])), axis=1)
                    IndexFactory.addConst('C_2X+g2=c_up', 1)
                    IndexFactory.addVar('g2', 1)
            else:
                rowi = sparseConcat(rowi, e_sparse(i, nInEquality, -1), 'h')
                A = sparseConcat(A, rowi, 'v')
                b = np.concatenate((b, np.array([c_low[i]])), axis=1)
                IndexFactory.addConst('C_3X-g3=c_low', 1)
                IndexFactory.addVar('g3', 1)

            #whatever the variable (g1,g2 or g3) >=0

        ##step two: adding x + x_u <= U and x - x_l >= L
        x_up_count = 0
        iVarsWithUpperBounds = [i for i in range(len(x_up))
                                    if x_up[i] < infinity]

        lengthOfRowToAdd = (IndexFactory.currentVarIndex +
                            len(iVarsWithUpperBounds))
        startInd = IndexFactory.currentVarIndex
        for i in iVarsWithUpperBounds:
                rowToAdd = e_sparse(i, lengthOfRowToAdd, 1)
                rowToAdd[0, startInd + x_up_count] = 1
                x_up_count += 1
                A = sparseConcat(A, rowToAdd, 'v')
                b = np.concatenate((b, np.array([x_up[i]])), axis=1)
                IndexFactory.addConst('x+x_u=x_up', 1)
                IndexFactory.addVar('x_u', 1)

        iVarsWithLowerBounds = [i for i in range(len(x_low))
                                            if x_low[i] > -infinity]

        lengthOfRowToAdd = (IndexFactory.currentVarIndex +
                                len(iVarsWithLowerBounds))

        startInd = IndexFactory.currentVarIndex
        x_low_count = 0
        #for i in x_low.keys():
        for i in iVarsWithLowerBounds:
                rowToAdd = e_sparse(i, lengthOfRowToAdd, 1)
                rowToAdd[0,  startInd + x_low_count] = -1
                x_low_count += 1
                A = sparseConcat(A, rowToAdd, 'v')
                b = np.concatenate((b, np.array([x_low[i]])), axis=1)
                IndexFactory.addConst('x-x_l=x_low', 1)
                IndexFactory.addVar('x_l', 1)

        if 'g1' in IndexFactory.varIndex.keys():
            g1_inds = IndexFactory.varIndex['g1']
            for i in range(len(g1_inds)):
                g1_i = g1_inds[i]
                rowToAdd = e_sparse(g1_i,
                            IndexFactory.currentVarIndex + len(g1_inds), 1)
                rowToAdd[0, IndexFactory.currentVarIndex + i] = 1
                A = sparseConcat(A, rowToAdd, 'v')
                indexInC = IndexFactory.constIndex['C_1X+g1=c_up'][i] - \
                                                                    nEquality
                b = np.concatenate((b, np.array([c_up[indexInC] -
                                    c_low[indexInC]])), axis=1)
                IndexFactory.addConst('g_1+g_u=c_up-c_low', 1)
                IndexFactory.addVar('g_u', 1)

        #(Augmented A) x = b
        for i in range(A.shape[0]):
            rowi = A.getrow(i).tocoo()
            model.addConstraint(rowi.nnz, np.array(rowi.col, np.int32),
                                np.array(rowi.data, 'd'), b[i], b[i])

        setPositive(model, 'g1')
        setPositive(model, 'g2')
        setPositive(model, 'g3')
        setPositive(model, 'x_u')
        setPositive(model, 'x_l')
        setPositive(model, 'g_u')

        #Gx = -c
        for i in range(nVar):
            rowi = G.getrow(i).tocoo()
            model.addConstraint(rowi.nnz, np.array(rowi.col, np.int32),
                        np.array(rowi.data, np.double), -c[i], -c[i])
        IndexFactory.addConst('Gx=-c', nVar)

        for i in range(nVar):
            model.setVariableLower(i, -infinity)

        #adding s^+    so we have: GX + s^+ = -c
        for i in range(nVar):
            model.addVariable(1, np.array(
                        [IndexFactory.constIndex['Gx=-c'][i]], np.int32),
                        np.array([1.], np.double), 0, infinity, 0)
        IndexFactory.addVar('s^+', nVar)

        #adding s^-    now: GX + s^+ - s^- = -c
        for i in range(nVar):
            model.addVariable(1, np.array([
                        IndexFactory.constIndex['Gx=-c'][i]], np.int32),
                        np.array([-1.], np.double), 0, infinity, 0)
        IndexFactory.addVar('s^-', nVar)

        # We can run the primal here to find a feasible point
        # to start the second phase
        # as in Wolfe. But in practice, doesn't seem like a good idea
        #s.loadProblem(model, 0)
        #s.primal()

        #adding extra 0's to G for variables other than x: g1, g2, x_u,...

        #adding rhs=0 for A^Ty constraints corresponding to
        # all variables except 'x'
        #These are the variables for which G.row(i) and c_i are zero
        #we don't know how many they are as, for example, we may or may not
        #have Cx<=c_up or Cx >= c_low or ...
        for i in range(A.shape[1] - nVar):
            model.addConstraint(0, np.array([], np.int32),
                                    np.array([], np.double), 0, 0)

        #adding -A^Ty_A
        for i in range(A.shape[0]):
            rowi = -A.getrow(i).tocoo()
            #print('rowi : ')
            #print(rowi.col)
            cols = rowi.col + IndexFactory.constIndex['Gx=-c'][0]
            #print(cols)
            model.addVariable(rowi.nnz, np.array(cols, np.int32),
                            np.array(rowi.data, 'd'), -infinity, infinity, 0)
        IndexFactory.addVar('y_A', A.shape[0])

        cl = np.array(range(IndexFactory.currentVarIndex +
                            A.shape[1]), np.int32)
        s.setComplementarityList(cl)

        #adding -z
        startRow = IndexFactory.constIndex['Gx=-c'][0]
        #TODO: check this range
        #for i in range(nVar, A.shape[1]):
        for i in range(nVar, A.shape[1]):
            model.addVariable(1, np.array([startRow + i], np.int32),
                            np.array([-1.], 'd'), 0, infinity, 0)
            IndexFactory.addVar('z', 1)
            compind = IndexFactory.getLastVarIndex()
            s.setComplement(i, compind)
            #cl[i] , cl[compind] = compind, i

        #setting the objective coefficients of s^+ to one,
        sPlusIndex = IndexFactory.varIndex['s^+']
        sMinusIndex = IndexFactory.varIndex['s^-']
        for i in sPlusIndex:
            model.setObjective(i, 1)

        #setting the objective coefficients of s^- to one,
        for i in sMinusIndex:
            model.setObjective(i, 1)

        s.loadProblem(model, 0)

        # I think that preSolve could be harmful here.
        # what happens to the complement variables?
        #s = s.preSolve(feasibilityTolerance = 10**-8)
        #s.setComplementarityList(cl)

        # This means that we want to use IClpSimplexPrimal_Wolfe
        # instead of ClpSimplexPrimal
        s.useCustomPrimal(1)

        #this means that we want to use cythons's Cywolfe
        #s.setPrimalColumnPivotAlgorithmToWolfe()
        p = WolfePivot(s, bucketSize=float(sys.argv[2]))

        #p = PositiveEdgeWolfePivot(s, bucketSize=float(sys.argv[2]))
        s.setPivotMethod(p)

        st = perf_counter()
        s.primal()
        print("CLP time : %g seconds" % (perf_counter() - st))

        x = s.getPrimalVariableSolution()
        print("sol = ")
        x = x[:nVar]
        #print(x)
        G = G.todense()
        print(0.5 * x * (x * G).T + np.dot(c, x) - self.objectiveOffset)
        return

        print('feasibility')
        print(getSolution(s, 'z'))
        print(getSolution(s, 'y_A'))
        #print((s.getPrimalSolution()[:nVar] * G).T)
        #print('*****')
        #print((s.getPrimalSolution()[:nVar] * G).T + getSolution(s, 's^+') -\
        #               getSolution(s, 's^-') -
        #               (getSolution(s, 'y_A') * A)[:nVar] - \
        #               getSolution(s, 'z')[:nVar] #+ np.dot(c, x))
        print('-c')
        print(-c)
        print(getSolution(s, 'z'))
        #print(getSolution(s, 'y_A') * A + getSolution(s, 'z'))
        print('s^+')
        print(getSolution(s, 's^+'))
        print('s^-')
        print(getSolution(s, 's^-'))

        x = s.getPrimalVariableSolution()
        print("sol = ")
        print(x)
        G = G.todense()
        print(G.shape)
        print(x * G)
        print(c)
        print(x)
        print(0.5 * x * (x * G).T + np.dot(c, x))


def getStat():
    import os
    d = '/Users/mehdi/Documents/work/benchmarks/qp/'
    for f in os.listdir(d):
        if f[-3:] == 'SIF':
            qp = QP()
            qp.fromQps(d + f)
            qp.convertToEqualityOnly()


def QPTest():
    qp = QP()
    start = perf_counter()
    qp.fromQps(sys.argv[1])
    qp.convertToEqualityOnly()
    if len(sys.argv) > 2:
        qp.WolfeEquality(sys.argv[2])
    else:
        qp.WolfeEquality()
    return
    r = perf_counter() - start
    if len(sys.argv) > 2:
        qp.Wolfe(sys.argv[2])
    else:
        qp.Wolfe()
    print('took %g seconds to read the problem' % r)
    print('took %g seconds to solve the problem' % (perf_counter() - start))
    print("done")

import sys
if __name__ == '__main__':
    QPTest()
    #cProfile.run('QPTest()')



cimport numpy as np
import numpy as np


cdef extern from "IOsiCuts.hpp":
    cdef cppclass CppOsiCuts "CppOsiCuts":
        void addColumnCut(int size, int* lowerBoundInds, double* lowerBoundElements,
                        int* upperBoundInds, double* upperBoundElements)
        void addRowCut(int size, int* indices, double* elements,
                   double lowerBound, double upperBound)
        void printCuts()
        int sizeRowCuts()
        int sizeColCuts()
        int sizeCuts()

    CppOsiCuts *new_CppOsiCuts "new CppOsiCuts" ()


cdef class CyOsiCuts:
    cdef CppOsiCuts* CppSelf
    cdef setCppSelf(self, CppOsiCuts* s)



from cylp.py.mip import NodeCompareBase

class SimpleNodeCompare(NodeCompareBase):

    def __init__(self):
        self.method = 'depth'  # For depth-First

    def compare(self, x, y):
        if x.numberUnsatisfied > y.numberUnsatisfied:
            return True
        elif x.numberUnsatisfied < y.numberUnsatisfied:
            return False
        if x.depth != y.depth:
            if self.method == 'depth':
                return x.depth < y.depth
            elif self.method == 'breadth':
                return x.depth > y.depth
        return x.breakTie(y)  # Breaking ties consistently

    def newSolution(self, model,
                            objectiveAtContinuous,
                            numberInfeasibilitiesAtContinuous):
        ''' This method is called after a solution is found
        in a node.
        '''
        self.method = 'breadth'

    def every1000Nodes(self, model, numberNodes):
        ''' This method is called each 1000 nodes for possible
        Changes in strategy
        '''
        return False



from __future__ import print_function
import sys
from time import perf_counter
import cProfile
import numpy as np
from cylp.cy import CyClpSimplex
from cylp.py.pivots import PositiveEdgePivot
from cylp.py.pivots.WolfePivot import WolfePivot
from cylp.py.pivots import LIFOPivot
from cylp.py.pivots import MostFrequentPivot
from cylp.py.pivots import DantzigPivot


def solve(filename, method):

    s = CyClpSimplex()
    s.readMps(filename)

    s.preSolve(feasibilityTolerance=10 ** -8)
    #s.useCustomPrimal(1)

    if method == 'd':
        pivot = DantzigPivot(s)
    elif method == 'l':
        pivot = LIFOPivot(s)
    elif method == 'm':
        pivot = MostFrequentPivot(s)
    elif method == 'p':
        pivot = PositiveEdgePivot(s)
    else:
        print('Unkown solution method.')
        sys.exit(1)

    s.setPivotMethod(pivot)

    #s.setPerturbation(50)

    start = perf_counter()
    s.primal()
    print('Problem solved in %g seconds.' % (perf_counter() - start))
    return s.objectiveValue


#    s.copyInIntegerInformation(np.array(s.nCols * [True], np.uint8))
#    #s.setInteger(100)
#
#    print("Solving relaxation")
#    cbcModel = s.getCbcModel()
#    from SimpleNodeCompare import SimpleNodeCompare
#    from CyCgl import CyCglGomory, CyCglClique, CyCglKnapsackCover
#    n = SimpleNodeCompare()
#    cbcModel.setNodeCompare(n)
##
#    gom = CyCglGomory(limit=150)
#    #gom.limit = 150
#    cbcModel.addCutGenerator(gom, name="Gomory")
#
#    #clq = CyCglClique()
#    #cbcModel.addCutGenerator(clq, name="Clique")
##
#
#    knap = CyCglKnapsackCover(maxInKnapsack=50)
#    cbcModel.addCutGenerator(knap, name="Knapsack")
#
#
#    cbcModel.branchAndBound()
#    print(cbcModel.primalVariableSolution)

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('Usage example (Dantzig pivot): PySolve file.mps d')
        sys.exit(1)
    solve(sys.argv[1], sys.argv[2])
    #cProfile.run('solve(sys.argv[1], sys.argv[2])')
